#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.canada import Canada
from tests.common import CommonCountryTests


class TestCanada(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Canada, with_subdiv_categories=True)

    def test_new_years_day(self):
        name = "New Year's Day"
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertGovernmentHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        for _, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name, holidays, (f"{year}-01-01" for year in self.full_range))

        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(name_observed, obs_dts)
        self.assertGovernmentHolidayName(name_observed, obs_dts)
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name_observed, holidays, obs_dts)
            self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        dts = (
            "1900-04-13",
            "1901-04-05",
            "1902-03-28",
            "1999-04-02",
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, self.full_range)
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, self.full_range)
        for _, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, self.full_range)

    def test_canada_day(self):
        name_1879 = "Dominion Day"
        name_1983 = "Canada Day"
        self.assertHolidayName(name_1879, (f"{year}-07-01" for year in range(1879, 1983)))
        self.assertHolidayName(name_1983, (f"{year}-07-01" for year in range(1983, self.end_year)))
        self.assertNoHolidayName(
            name_1879, range(self.start_year, 1879), range(1983, self.end_year)
        )
        self.assertNoHolidayName(name_1983, range(self.start_year, 1983))
        self.assertNoHoliday(f"{year}-07-01" for year in range(self.start_year, 1879))

        obs_dts_sat = (
            "2000-07-03",
            "2006-07-03",
            "2017-07-03",
            "2023-07-03",
        )
        obs_dts_sun = (
            "2001-07-02",
            "2007-07-02",
            "2012-07-02",
            "2018-07-02",
        )
        name_observed = f"{name_1983} (observed)"
        self.assertNoHoliday(obs_dts_sat, obs_dts_sun)
        self.assertGovernmentHolidayName(name_observed, obs_dts_sat, obs_dts_sun)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_sat, obs_dts_sun)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "AB" | "BC" | "QC":
                    self.assertHolidayName(name_observed, holidays, obs_dts_sun)
                    self.assertNoHoliday(holidays, obs_dts_sat)
                case "NL" | "PE" | "SK" | "YT":
                    self.assertHoliday(holidays, obs_dts_sat, obs_dts_sun)
                case _:
                    self.assertNoHoliday(holidays, obs_dts_sat, obs_dts_sun)
            self.assertNoNonObservedHoliday(
                self.subdiv_holidays_non_observed[subdiv], obs_dts_sat, obs_dts_sun
            )

    def test_labor_day(self):
        name = "Labour Day"
        self.assertNoHolidayName(name, range(self.start_year, 1894))
        self.assertHolidayName(name, range(1894, self.end_year))
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 1894))
        self.assertGovernmentHolidayName(name, range(1894, self.end_year))

        dts = (
            "1894-09-03",
            "1900-09-03",
            "1999-09-06",
            "2000-09-04",
            "2014-09-01",
            "2015-09-07",
            "2018-09-03",
            "2019-09-02",
            "2020-09-07",
            "2021-09-06",
            "2022-09-05",
            "2023-09-04",
        )
        self.assertHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, dts)
        for _, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name, holidays, dts)
            self.assertHolidayName(name, holidays, range(1894, self.end_year))
            self.assertNoHolidayName(name, holidays, range(self.start_year, 1894))

    def test_christmas_day(self):
        name = "Christmas Day"
        name_observed = f"{name} (observed)"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        for _, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(name, holidays, (f"{year}-12-25" for year in self.full_range))

        obs_dts_sat = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        obs_dts_sun_without_boxing = (
            "2005-12-26",
            "2011-12-26",
            "2016-12-26",
            "2022-12-26",
        )
        obs_dts_sun_with_boxing = (
            "2005-12-27",
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(name_observed, obs_dts_sat, obs_dts_sun_without_boxing)
        self.assertNoHoliday(obs_dts_sun_with_boxing)
        self.assertNoNonObservedHoliday(
            obs_dts_sat, obs_dts_sun_with_boxing, obs_dts_sun_without_boxing
        )
        self.assertGovernmentHolidayName(name_observed, obs_dts_sat, obs_dts_sun_with_boxing)
        self.assertNoGovernmentNonObservedHoliday(obs_dts_sat, obs_dts_sun_with_boxing)
        self.assertOptionalHolidayName(name_observed, obs_dts_sat, obs_dts_sun_with_boxing)
        self.assertNoOptionalNonObservedHoliday(obs_dts_sat, obs_dts_sun_with_boxing)
        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertHolidayName(
                name_observed, holidays, obs_dts_sat, obs_dts_sun_without_boxing
            )
            self.assertNoHoliday(holidays, obs_dts_sun_with_boxing)
            self.assertNoNonObservedHoliday(
                self.subdiv_holidays_non_observed[subdiv], obs_dts_sat, obs_dts_sun_with_boxing
            )

    def test_victoria_day(self):
        name = "Victoria Day"
        name_observed = f"{name} (observed)"

        dts = (
            "1953-05-18",
            "2000-05-22",
            "2010-05-24",
            "2018-05-21",
            "2019-05-20",
            "2020-05-18",
            "2021-05-24",
            "2022-05-23",
            "2023-05-22",
        )
        obs_dts = (
            "1903-05-25",
            "1908-05-25",
            "1914-05-25",
            "1925-05-25",
            "1931-05-25",
            "1936-05-25",
            "1942-05-25",
        )
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, dts)
        self.assertGovernmentHolidayName(name, (f"{year}-05-24" for year in range(1901, 1953)))
        self.assertGovernmentHolidayName(name, range(1953, self.end_year))
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 1901))
        self.assertGovernmentHolidayName(name_observed, obs_dts)
        self.assertNoGovernmentNonObservedHoliday(obs_dts)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AB", "BC", "MB", "NT", "NU", "ON", "SK", "YT"}:
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-24" for year in range(1901, 1953))
                )
                self.assertHolidayName(name, holidays, range(1953, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1901))
                self.assertHolidayName(name_observed, holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_national_day_for_truth_and_reconciliation(self):
        name = "National Day for Truth and Reconciliation"
        name_observed = f"{name} (observed)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name, (f"{year}-09-30" for year in range(2021, self.end_year))
        )
        self.assertNoGovernmentHolidayName(name, range(self.start_year, 2021))

        obs_dts = (
            "2023-10-02",
            "2028-10-02",
            "2029-10-01",
        )
        self.assertGovernmentHolidayName(name_observed, obs_dts)
        self.assertNoGovernmentNonObservedHoliday(obs_dts)

        subdiv_start_years = {
            "BC": 2023,
            "MB": 2024,
            "NT": 2022,
            "NU": 2022,
            "PE": 2022,
            "YT": 2023,
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            if start_year := subdiv_start_years.get(subdiv):
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-30" for year in range(start_year, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, start_year))
            else:
                self.assertNoHolidayName(name, holidays)
            self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)

        obs_dts = (
            "2028-10-02",
            "2029-10-01",
        )
        self.assertSubdivMbHolidayName(name_observed, obs_dts)
        self.assertNoSubdivMbNonObservedHoliday(obs_dts)

        self.assertSubdivAbOptionalHolidayName(
            name, (f"{year}-09-30" for year in range(2021, self.end_year))
        )
        self.assertNoSubdivAbOptionalHolidayName(name, range(self.start_year, 2021))

    def test_thanksgiving_day(self):
        name_1921 = "Armistice Day"
        name_1931 = "Thanksgiving Day"

        dts_1921 = (
            # By Statute.
            "1921-11-07",
            "1922-11-06",
            "1923-11-12",
            "1924-11-10",
            "1925-11-09",
            "1926-11-08",
            "1927-11-07",
            "1928-11-12",
            "1929-11-11",
            "1930-11-10",
        )
        self.assertNoHolidayName(name_1921)
        self.assertGovernmentHolidayName(name_1921, dts_1921)
        self.assertNoGovernmentHolidayName(
            name_1921, range(self.start_year, 1921), range(1931, self.end_year)
        )

        dts_1931 = (
            # By Proclamation.
            "1931-10-12",
            "1935-10-24",
            # Post-Proclamation of 1957.
            "1990-10-08",
            "1999-10-11",
            "2000-10-09",
            "2013-10-14",
            "2018-10-08",
            "2019-10-14",
            "2020-10-12",
            "2021-10-11",
            "2022-10-10",
            "2023-10-09",
        )
        self.assertNoHolidayName(name_1931)
        self.assertGovernmentHolidayName(name_1931, dts_1931)
        self.assertGovernmentHolidayName(name_1931, range(1931, self.end_year))
        self.assertNoGovernmentHolidayName(name_1931, range(self.start_year, 1931))

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AB", "BC", "MB", "NT", "NU", "ON", "QC", "SK", "YT"}:
                self.assertHolidayName(name_1931, holidays, dts_1931)
                self.assertHolidayName(name_1921, holidays, range(1921, 1931))
                self.assertHolidayName(name_1931, holidays, range(1931, self.end_year))
                self.assertNoHolidayName(
                    name_1921, holidays, range(self.start_year, 1921), range(1931, self.end_year)
                )
                self.assertNoHolidayName(name_1931, holidays, range(self.start_year, 1931))
            else:
                self.assertNoHolidayName(name_1921, holidays, dts_1921)
                self.assertNoHolidayName(name_1931, holidays, dts_1931)

        for subdiv in ("NB", "NL"):
            self.assertHolidayName(name_1921, self.subdiv_optional_holidays[subdiv], dts_1921)
            self.assertHolidayName(name_1931, self.subdiv_optional_holidays[subdiv], dts_1931)

    def test_remembrance_day(self):
        name = "Remembrance Day"
        name_observed = f"{name} (observed)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name, (f"{year}-11-11" for year in range(1931, self.end_year))
        )
        self.assertNoGovernmentHolidayName(name, range(1900, 1931))

        obs_dts = (
            "2006-11-13",
            "2007-11-12",
            "2012-11-12",
            "2017-11-13",
            "2018-11-12",
            "2023-11-13",
        )
        self.assertGovernmentHolidayName(name_observed, obs_dts)
        self.assertNoGovernmentNonObservedHoliday(obs_dts)

        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"AB", "BC", "NB", "NL", "NS", "NT", "NU", "PE", "SK", "YT"}:
                start_year = 1981 if subdiv == "NS" else 1931
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-11" for year in range(start_year, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1900, start_year))
            else:
                self.assertNoHolidayName(name, holidays)

            if subdiv in {"AB", "NL", "NS", "PE", "SK", "YT"}:
                self.assertHolidayName(name_observed, holidays, obs_dts)
                self.assertNoNonObservedHoliday(self.subdiv_holidays_non_observed[subdiv], obs_dts)
            else:
                self.assertNoHoliday(holidays, obs_dts)

        self.assertSubdivMbOptionalHolidayName(
            name, (f"{year}-11-11" for year in range(1931, self.end_year))
        )

    def test_boxing_day(self):
        name = "Boxing Day"
        name_observed = f"{name} (observed)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        self.assertOptionalHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        obs_dts = (
            "2004-12-28",
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertOptionalHolidayName(name_observed, obs_dts)
        self.assertNoOptionalNonObservedHoliday(obs_dts)

        obs_dts = (
            "2004-12-28",
            "2010-12-28",
            "2021-12-28",
        )
        self.assertSubdivOnHolidayName(name_observed, obs_dts)
        self.assertNoSubdivOnNonObservedHoliday(obs_dts)

        for subdiv in ("AB", "NB", "NL"):
            self.assertHolidayName(
                name,
                self.subdiv_optional_holidays[subdiv],
                (f"{year}-12-26" for year in self.full_range),
            )

    def test_family_day(self):
        start_years = {
            "AB": 1990,
            "BC": 2019,
            "MB": 2008,
            "NB": 2018,
            "NS": 2015,
            "ON": 2008,
            "PE": 2010,
            "SK": 2007,
        }
        dts = (
            "1990-02-19",
            "1991-02-18",
            "1992-02-17",
            "1993-02-15",
            "1994-02-21",
            "1995-02-20",
            "1996-02-19",
            "1997-02-17",
            "1998-02-16",
            "1999-02-15",
            "2000-02-21",
            "2001-02-19",
            "2002-02-18",
            "2003-02-17",
            "2004-02-16",
            "2005-02-21",
            "2006-02-20",
            "2007-02-19",
            "2008-02-18",
            "2009-02-16",
            "2010-02-15",
            "2011-02-21",
            "2012-02-20",
            "2013-02-18",
            "2014-02-17",
            "2015-02-16",
            "2016-02-15",
            "2017-02-20",
            "2018-02-19",
            "2019-02-18",
            "2020-02-17",
            "2021-02-15",
            "2022-02-21",
            "2023-02-20",
        )
        subdiv_names = {
            "MB": "Louis Riel Day",
            "NS": "Heritage Day",
            "PE": "Islander Day",
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            name = subdiv_names.get(subdiv, "Family Day")
            for year, dt in enumerate(dts, 1990):
                if subdiv in start_years and year >= start_years[subdiv]:
                    self.assertHolidayName(name, holidays, dt)
                else:
                    self.assertNoHoliday(holidays, dt)
        self.assertNoHoliday(dts)
        self.assertNoHolidayName("Family Day")
        for name in subdiv_names.values():
            self.assertNoHolidayName(name)

        self.assertSubdivBcHoliday(
            "2013-02-11",
            "2014-02-10",
            "2015-02-09",
            "2016-02-08",
            "2017-02-13",
            "2018-02-12",
        )
        self.assertSubdivPeHoliday("2009-02-09")

    def test_easter_monday(self):
        name = "Easter Monday"
        dts = (
            "1900-04-16",
            "1901-04-08",
            "1902-03-31",
            "1999-04-05",
            "2000-04-24",
            "2010-04-05",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertNoGovernmentHoliday(dts)
        self.assertNoGovernmentHolidayName(name)

        for subdiv, holidays in self.subdiv_holidays.items():
            self.assertNoHoliday(holidays, dts)
            self.assertNoHolidayName(name, holidays)

            if subdiv in {"AB", "QC"}:
                self.assertHolidayName(name, self.subdiv_optional_holidays[subdiv], dts)

    def test_civic_holiday_ab(self):
        name = "Heritage Day"
        self.assertNoSubdivAbHolidayName(name)
        dts = (
            "1974-08-05",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivAbOptionalHolidayName(name, dts)
        self.assertSubdivAbOptionalHolidayName(name, range(1974, self.end_year))
        self.assertNoSubdivAbOptionalHolidayName(name, range(self.start_year, 1974))

    def test_civic_holiday_bc(self):
        name = "British Columbia Day"
        dts = (
            "1974-08-05",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivBcHolidayName(name, dts)
        self.assertSubdivBcHolidayName(name, range(1974, self.end_year))
        self.assertNoSubdivBcHolidayName(name, range(self.start_year, 1974))

    def test_civic_holiday_mb(self):
        old_name = "Civic Holiday"
        new_name = "Terry Fox Day"
        self.assertNoSubdivMbHolidayName(old_name)
        self.assertNoSubdivMbHolidayName(new_name)

        dts_old = (
            "1900-08-06",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2014-08-04",
        )
        dts_new = (
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts_old, dts_new)
        self.assertNoHolidayName(old_name)
        self.assertNoHolidayName(new_name)

        self.assertSubdivMbOptionalHolidayName(old_name, dts_old)
        self.assertSubdivMbOptionalHolidayName(old_name, range(1900, 2015))
        self.assertNoSubdivMbOptionalHolidayName(
            old_name, range(self.start_year, 1900), range(2015, self.end_year)
        )

        self.assertSubdivMbOptionalHolidayName(new_name, dts_new)
        self.assertSubdivMbOptionalHolidayName(new_name, range(2015, self.end_year))
        self.assertNoSubdivMbOptionalHolidayName(new_name, range(self.start_year, 2015))

    def test_civic_holiday_nb(self):
        name = "New Brunswick Day"
        dts = (
            "1975-08-04",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNbHolidayName(name, dts)
        self.assertSubdivNbHolidayName(name, range(1975, self.end_year))
        self.assertNoSubdivNbHolidayName(name, range(self.start_year, 1975))

    def test_civic_holiday_ns(self):
        name = "Natal Day"
        self.assertNoSubdivNsHolidayName(name)
        dts = (
            "1996-08-05",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNsOptionalHolidayName(name, dts)
        self.assertSubdivNsOptionalHolidayName(name, range(1996, self.end_year))
        self.assertNoSubdivNsOptionalHolidayName(name, range(self.start_year, 1996))

    def test_civic_holiday_nt_nu_on_sk(self):
        name = "Civic Holiday"
        name_sk = "Saskatchewan Day"

        dts = (
            "1900-08-06",
            "1999-08-02",
            "2000-08-07",
            "2010-08-02",
            "2015-08-03",
            "2020-08-03",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)

        self.assertSubdivNtHolidayName(name, dts)
        self.assertSubdivNtHolidayName(name, range(1900, self.end_year))
        self.assertNoSubdivNtHolidayName(name, range(self.start_year, 1900))

        self.assertSubdivNuHolidayName(name, dts)
        self.assertSubdivNuHolidayName(name, range(1900, self.end_year))
        self.assertNoSubdivNuHolidayName(name, range(self.start_year, 1900))

        self.assertSubdivOnOptionalHolidayName(name, dts)
        self.assertSubdivOnOptionalHolidayName(name, range(1900, self.end_year))
        self.assertNoSubdivOnOptionalHolidayName(name, range(self.start_year, 1900))

        self.assertSubdivSkHolidayName(name_sk, dts)
        self.assertSubdivSkHolidayName(name_sk, range(1900, self.end_year))
        self.assertNoSubdivSkHolidayName(name_sk, range(self.start_year, 1900))

    def test_memorial_day(self):
        name = "Memorial Day"
        self.assertNoHolidayName(name)
        self.assertSubdivNlHolidayName(
            name, (f"{year}-07-01" for year in range(1917, self.end_year))
        )
        self.assertNoSubdivNlHolidayName(name, range(1900, 1917))

    def test_st_patricks_day(self):
        name = "Saint Patrick's Day"
        self.assertNoSubdivNlHolidayName(name)
        dts = (
            "1900-03-19",
            "1999-03-15",
            "2000-03-20",
            "2012-03-19",
            "2013-03-18",
            "2014-03-17",
            "2015-03-16",
            "2016-03-14",
            "2020-03-16",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNlOptionalHolidayName(name, dts)
        self.assertSubdivNlOptionalHolidayName(name, range(1900, self.end_year))
        self.assertNoSubdivNlOptionalHolidayName(name, range(self.start_year, 1900))

    def test_st_georges_day(self):
        name = "Saint George's Day"
        self.assertNoSubdivNlHolidayName(name)
        dts = (
            "1990-04-23",
            "1999-04-26",
            "2010-04-19",
            "2016-04-25",
            "2020-04-20",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNlOptionalHolidayName(name, dts)
        self.assertSubdivNlOptionalHolidayName(name, range(1990, self.end_year))
        self.assertNoSubdivNlOptionalHolidayName(name, range(self.start_year, 1990))

    def test_discovery_day_nl(self):
        name = "Discovery Day"
        self.assertNoSubdivNlHolidayName(name)
        dts = (
            "1997-06-23",
            "1999-06-21",
            "2000-06-26",
            "2010-06-21",
            "2016-06-27",
            "2020-06-22",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNlOptionalHolidayName(name, dts)
        self.assertSubdivNlOptionalHolidayName(name, range(1997, self.end_year))
        self.assertNoSubdivNlOptionalHolidayName(name, range(self.start_year, 1997))

    def test_orangemans_day(self):
        name = "Orangemen's Day"
        self.assertNoSubdivNlHolidayName(name)
        dts = (
            "1900-07-09",
            "1999-07-12",
            "2000-07-10",
            "2010-07-12",
            "2016-07-11",
            "2020-07-13",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivNlOptionalHolidayName(name, dts)
        self.assertSubdivNlOptionalHolidayName(name, range(1900, self.end_year))
        self.assertNoSubdivNlOptionalHolidayName(name, range(self.start_year, 1900))

    def test_discovery_day_yt(self):
        name = "Discovery Day"
        dts = (
            "1912-08-19",
            "1999-08-16",
            "2000-08-21",
            "2006-08-21",
            "2016-08-15",
            "2020-08-17",
        )
        self.assertNoHoliday(dts)
        self.assertNoHolidayName(name)
        self.assertSubdivYtHolidayName(name, dts)
        self.assertSubdivYtHolidayName(name, range(1912, self.end_year))
        self.assertNoSubdivYtHolidayName(name, range(self.start_year, 1912))

    def test_national_aboriginal_day(self):
        name = "National Aboriginal Day"
        self.assertSubdivNtHolidayName(
            name, (f"{year}-06-21" for year in range(2001, self.end_year))
        )
        self.assertNoSubdivNtHolidayName(name, range(self.start_year, 2001))
        self.assertSubdivYtHolidayName(
            name, (f"{year}-06-21" for year in range(2017, self.end_year))
        )
        self.assertNoSubdivYtHolidayName(name, range(self.start_year, 2017))
        self.assertNoHoliday(f"{year}-06-21" for year in range(1996, self.end_year))
        self.assertNoHolidayName(name)

    def test_nunavut_day(self):
        name = "Nunavut Day"
        self.assertNoHolidayName(name)
        self.assertNoHoliday(f"{year}-07-09" for year in range(2001, self.end_year))
        self.assertSubdivNuHolidayName(
            name, (f"{year}-07-09" for year in range(2020, self.end_year))
        )
        self.assertNoSubdivNuHolidayName(name, range(self.start_year, 2020))
        self.assertNoSubdivNuOptionalHoliday("1999-07-09", "2000-07-09", "2020-07-09")
        self.assertSubdivNuOptionalHolidayName(
            name,
            "2000-04-01",
            (f"{year}-07-09" for year in range(2001, 2020)),
        )
        self.assertNoSubdivNuOptionalHolidayName(
            name, range(self.start_year, 2000), range(2020, self.end_year)
        )

    def test_national_patriots_day(self):
        name = "National Patriots' Day"
        self.assertNoHolidayName(name)
        self.assertSubdivQcHolidayName(
            name,
            "2010-05-24",
            "2015-05-18",
            "2020-05-18",
            "2021-05-24",
            "2022-05-23",
        )
        self.assertSubdivQcHolidayName(name, range(2003, self.end_year))
        self.assertNoSubdivQcHolidayName(name, range(self.start_year, 2003))

    def test_st_jean_baptiste_day(self):
        name = "Saint Jean Baptiste Day"
        self.assertNoHolidayName(name)
        self.assertSubdivQcHoliday(f"{year}-06-24" for year in range(1925, self.end_year))
        self.assertNoSubdivQcHoliday(f"{year}-06-24" for year in range(self.start_year, 1925))
        self.assertNoHoliday(f"{year}-06-24" for year in range(1925, self.end_year))
        self.assertSubdivQcHoliday("2001-06-25")
        self.assertNoSubdivQcNonObservedHoliday("2001-06-25")

    def test_yukon_heritage_day(self):
        name = "Heritage Day"
        self.assertNoHolidayName(name)
        self.assertNoSubdivYtHolidayName(name)
        dts = (
            "2017-02-24",
            "2018-02-23",
            "2019-02-22",
            "2020-02-21",
            "2021-02-26",
            "2022-02-25",
        )
        self.assertSubdivYtOptionalHolidayName(name, dts)
        self.assertSubdivYtOptionalHolidayName(name, range(1976, self.end_year))
        self.assertNoSubdivYtOptionalHolidayName(name, range(self.start_year, 1976))

    def test_queens_funeral(self):
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"BC", "NB", "NL", "NS", "PE", "YT"}:
                self.assertHoliday(holidays, "2022-09-19")
            else:
                self.assertNoHoliday(holidays, "2022-09-19")

    def test_public_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-07-01", "Canada Day"),
            ("2022-09-05", "Labour Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Christmas Day (observed)"),
        )

    def test_government_2022(self):
        self.assertGovernmentHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-23", "Victoria Day"),
            ("2022-07-01", "Canada Day"),
            ("2022-09-05", "Labour Day"),
            ("2022-09-30", "National Day for Truth and Reconciliation"),
            ("2022-10-10", "Thanksgiving Day"),
            ("2022-11-11", "Remembrance Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_optional_2022(self):
        self.assertOptionalHolidaysInYear(
            2022,
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_all_holidays_present(self):
        y_2022 = set()
        for subdiv in Canada.subdivisions:
            y_2022.update(Canada(years=2022, subdiv=subdiv, observed=False).values())
        all_h = {  # Holidays names in their chronological order.
            "New Year's Day",
            "Family Day",
            "Heritage Day",
            "Islander Day",
            "Louis Riel Day",
            "Good Friday",
            "National Patriots' Day",
            "Victoria Day",
            "National Aboriginal Day",
            "Saint Jean Baptiste Day",
            "Canada Day",
            "Canada Day; Memorial Day",
            "Nunavut Day",
            "British Columbia Day",
            "Civic Holiday",
            "New Brunswick Day",
            "Saskatchewan Day",
            "Discovery Day",
            "Labour Day",
            "Funeral of Her Majesty the Queen Elizabeth II",
            "National Day for Truth and Reconciliation",
            "Thanksgiving Day",
            "Remembrance Day",
            "Christmas Day",
            "Boxing Day",
        }

        self.assertEqual(
            all_h,
            y_2022,
            f"missing: {all_h - y_2022 or 'no'}, extra: {y_2022 - all_h or 'no'}",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-02-21", "Family Day; Heritage Day; Islander Day; Louis Riel Day"),
            ("2022-02-25", "Heritage Day"),
            ("2022-03-14", "Saint Patrick's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "Saint George's Day"),
            ("2022-05-23", "National Patriots' Day; Victoria Day"),
            ("2022-06-21", "National Aboriginal Day"),
            ("2022-06-24", "Saint Jean Baptiste Day"),
            ("2022-06-27", "Discovery Day"),
            ("2022-07-01", "Canada Day; Memorial Day"),
            ("2022-07-09", "Nunavut Day"),
            ("2022-07-11", "Orangemen's Day"),
            (
                "2022-08-01",
                "British Columbia Day; Civic Holiday; Heritage Day; Natal Day; "
                "New Brunswick Day; Saskatchewan Day; Terry Fox Day",
            ),
            ("2022-08-15", "Discovery Day"),
            ("2022-09-05", "Labour Day"),
            ("2022-09-19", "Funeral of Her Majesty the Queen Elizabeth II"),
            ("2022-09-30", "National Day for Truth and Reconciliation"),
            ("2022-10-10", "Thanksgiving Day"),
            ("2022-11-11", "Remembrance Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed)"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_ar(self):
        self.assertLocalizedHolidays(
            "ar",
            ("2022-01-01", "يوم السنة الجديدة"),
            ("2022-01-03", "يوم السنة الجديدة (يوم تعويضي)"),
            ("2022-02-21", "يوم التراث; يوم الجزيرة; يوم العائلة; يوم لويس رئيل"),
            ("2022-02-25", "يوم التراث"),
            ("2022-03-14", "عيد القديس باتريك"),
            ("2022-04-15", "الجمعة العظيمة"),
            ("2022-04-18", "عيد الفصح الاثنين"),
            ("2022-04-25", "عيد القديس جورج"),
            ("2022-05-23", "يوم الوطنيين; يوم فيكتوريا"),
            ("2022-06-21", "اليوم الوطني للسكان الأصليين"),
            ("2022-06-24", "عيد القديس جان بابتيست"),
            ("2022-06-27", "يوم الاكتشاف"),
            ("2022-07-01", "يوم الذكرى; يوم كندا"),
            ("2022-07-09", "يوم نونافوت"),
            ("2022-07-11", "يوم رجال البرتقال"),
            (
                "2022-08-01",
                "عطلة المدنية; يوم التأسيس; يوم التراث; يوم تيري فوكس; يوم ساسكاتشوان; "
                "يوم كولومبيا البريطانية; يوم نيو برونزويك",
            ),
            ("2022-08-15", "يوم الاكتشاف"),
            ("2022-09-05", "عيد العمال"),
            ("2022-09-19", "جنازة جلالة الملكة اليزابيث الثانية"),
            ("2022-09-30", "اليوم الوطني للحقيقة والمصالحة"),
            ("2022-10-10", "عيد الشكر"),
            ("2022-11-11", "يوم الذكرى"),
            ("2022-12-25", "عيد الميلاد"),
            ("2022-12-26", "عيد الميلاد (يوم تعويضي); يوم الملاكمة"),
            ("2022-12-27", "عيد الميلاد (يوم تعويضي)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-03", "New Year's Day (observed)"),
            ("2022-02-21", "Family Day; Heritage Day; Islander Day; Louis Riel Day"),
            ("2022-02-25", "Heritage Day"),
            ("2022-03-14", "Saint Patrick's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-25", "Saint George's Day"),
            ("2022-05-23", "National Patriots' Day; Victoria Day"),
            ("2022-06-21", "National Aboriginal Day"),
            ("2022-06-24", "Saint John the Baptist Day"),
            ("2022-06-27", "Discovery Day"),
            ("2022-07-01", "Canada Day; Memorial Day"),
            ("2022-07-09", "Nunavut Day"),
            ("2022-07-11", "Orangemen's Day"),
            (
                "2022-08-01",
                "British Columbia Day; Civic Holiday; Heritage Day; Natal Day; New Brunswick Day; "
                "Saskatchewan Day; Terry Fox Day",
            ),
            ("2022-08-15", "Discovery Day"),
            ("2022-09-05", "Labor Day"),
            ("2022-09-19", "Funeral of Her Majesty the Queen Elizabeth II"),
            ("2022-09-30", "National Day for Truth and Reconciliation"),
            ("2022-10-10", "Thanksgiving Day"),
            ("2022-11-11", "Remembrance Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day; Christmas Day (observed)"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2022-01-01", "Jour de l'an"),
            ("2022-01-03", "Jour de l'an (Observé)"),
            (
                "2022-02-21",
                "Fête de la famille; Fête des Insulaires; Fête du Patrimoine; Journée Louis Riel",
            ),
            ("2022-02-25", "Fête du Patrimoine"),
            ("2022-03-14", "Fête de la Saint-Patrick"),
            ("2022-04-15", "Vendredi saint"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-04-25", "Fête de la Saint-Georges"),
            ("2022-05-23", "Fête de la Reine; Journée nationale des patriotes"),
            ("2022-06-21", "Journée nationale des Autochtones"),
            ("2022-06-24", "Fête nationale du Québec"),
            ("2022-06-27", "Jour de la Découverte"),
            ("2022-07-01", "Fête du Canada; Jour de mémorial"),
            ("2022-07-09", "Jour du Nunavut"),
            ("2022-07-11", "Journée des Orangistes"),
            (
                "2022-08-01",
                "Fête du Patrimoine; Jour de la Colombie Britannique; Jour de la Fondation; "
                "Jour du Nouveau Brunswick; Jour du Saskatchewan; Journée Terry Fox; "
                "Premier lundi d'août",
            ),
            ("2022-08-15", "Jour de la Découverte"),
            ("2022-09-05", "Fête du Travail"),
            ("2022-09-19", "Funéraire de sa majesté la reine Elizabeth II"),
            ("2022-09-30", "Journée nationale de la vérité et de la réconciliation"),
            ("2022-10-10", "Action de grâce"),
            ("2022-11-11", "Jour du Souvenir"),
            ("2022-12-25", "Jour de Noël"),
            ("2022-12-26", "Boxing Day; Jour de Noël (Observé)"),
            ("2022-12-27", "Jour de Noël (Observé)"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-01-03", "ชดเชยวันขึ้นปีใหม่"),
            (
                "2022-02-21",
                "วันครอบครัว; วันชาวเกาะ (พรินซ์เอดเวิร์ดไอแลนด์); วันมรดก; วันหลุยส์เรียล (แมนิโทบา)",
            ),
            ("2022-02-25", "วันมรดก"),
            ("2022-03-14", "วันเซนต์แพทริก (นิวฟันด์แลนด์และแลบราดอร์)"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-04-25", "วันเซนต์จอร์จ (นิวฟันด์แลนด์และแลบราดอร์)"),
            ("2022-05-23", "วันรำลึกกลุ่มแปตรีออต (ควิเบก); วันวิคตอเรีย"),
            ("2022-06-21", "วันชนพื้นเมือง (นอร์ทเวสต์เทร์ริทอรีส์)"),
            ("2022-06-24", "วันแซงต์-ฌ็อง-บาติสต์ (ควิเบก)"),
            ("2022-06-27", "วันค้นพบ"),
            ("2022-07-01", "วันชาติแคนาดา; วันรำลึก (นิวฟันด์แลนด์และแลบราดอร์)"),
            ("2022-07-09", "วันนูนาวุต"),
            ("2022-07-11", "วันออเรนจ์เมนส์"),
            (
                "2022-08-01",
                "วันซัสแคตเชวัน; วันนิวบรันสวิก; วันบริติชโคลัมเบีย; วันมรดก; วันสถาปนา; "
                "วันหยุดราชการ; วันเทร์รี ฟอกซ์",
            ),
            ("2022-08-15", "วันค้นพบ"),
            ("2022-09-05", "วันแรงงาน"),
            ("2022-09-19", "พระราชพิธีพระบรมศพของสมเด็จพระราชินีนาถเอลิซาเบธที่ 2"),
            ("2022-09-30", "วันชาติแห่งความจริงและการปรองดอง"),
            ("2022-10-10", "วันขอบคุณพระเจ้า"),
            ("2022-11-11", "วันรำลึก"),
            ("2022-12-25", "วันคริสต์มาส"),
            ("2022-12-26", "ชดเชยวันคริสต์มาส; วันเปิดกล่องของขวัญ"),
            ("2022-12-27", "ชดเชยวันคริสต์มาส"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.burundi import Burundi
from tests.common import CommonCountryTests


class TestBurundi(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Burundi)

    def test_new_year_day(self):
        name = "Jour de l'an"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_unity_day(self):
        name = "Fête de l'Unité"
        self.assertHolidayName(name, (f"{year}-02-05" for year in range(1992, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1992))
        obs_dts = (
            "2012-02-06",
            "2017-02-06",
            "2023-02-06",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_commemoration_of_the_assassination_of_president_cyprien_ntaryamira(self):
        name = "Commémoration de l'Assassinat du Président Cyprien Ntaryamira"
        self.assertHolidayName(name, (f"{year}-04-06" for year in range(1995, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1995))
        obs_dts = (
            "2008-04-07",
            "2014-04-07",
            "2025-04-07",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_international_labor_day(self):
        name = "Fête Internationale du Travail"
        self.assertHolidayName(name, (f"{year}-05-01" for year in self.full_range))
        obs_dts = (
            "2011-05-02",
            "2016-05-02",
            "2033-05-02",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_ascension_day(self):
        name = "Jour de l'Ascension"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_commemoration_of_the_death_of_president_pierre_nkurunziza(self):
        name = (
            "Journée Nationale du Patriotisme et Commémoration de la Mort "
            "du Président Pierre Nkurunziza"
        )
        self.assertHolidayName(name, (f"{year}-06-08" for year in range(2022, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2022))
        obs_dts = (
            "2025-06-09",
            "2031-06-09",
            "2036-06-09",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_independence_day(self):
        name = "Anniversaire de l'Indépendance"
        self.assertHolidayName(name, (f"{year}-07-01" for year in self.full_range))
        obs_dts = (
            "2012-07-02",
            "2018-07-02",
            "2029-07-02",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_assumption_day(self):
        name = "Assomption"
        self.assertHolidayName(name, (f"{year}-08-15" for year in self.full_range))
        obs_dts = (
            "2010-08-16",
            "2021-08-16",
            "2027-08-16",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_commemoration_of_the_assassination_of_national_hero_prince_louis_rwagasore(self):
        name = "Commémoration de l'Assassinat du Héros National, le Prince Louis Rwagasore"
        self.assertHolidayName(name, (f"{year}-10-13" for year in self.full_range))
        obs_dts = (
            "2013-10-14",
            "2019-10-14",
            "2024-10-14",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_commemoration_of_the_assassination_of_president_melchior_ndadaye(self):
        name = "Commémoration de l'Assassinat du Président Melchior Ndadaye"
        self.assertHolidayName(name, (f"{year}-10-21" for year in range(1994, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1994))
        obs_dts = (
            "2012-10-22",
            "2018-10-22",
            "2029-10-22",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_all_saints_day(self):
        name = "Toussaint"
        self.assertHolidayName(name, (f"{year}-11-01" for year in self.full_range))
        obs_dts = (
            "2015-11-02",
            "2020-11-02",
            "2026-11-02",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Noël"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2016-12-26",
            "2022-12-26",
            "2033-12-26",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_fitr(self):
        name = "Aid-El-Fithr"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2020-05-25",
            "2025-03-31",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_eid_al_adha(self):
        name = "Aid-El-Adha"
        self.assertHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)
        obs_dts = (
            "2019-08-12",
            "2024-06-17",
        )
        self.assertHolidayName(f"{name} (observé)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Jour de l'an"),
            ("2024-02-05", "Fête de l'Unité"),
            ("2024-04-06", "Commémoration de l'Assassinat du Président Cyprien Ntaryamira"),
            ("2024-04-10", "Aid-El-Fithr"),
            ("2024-05-01", "Fête Internationale du Travail"),
            ("2024-05-09", "Jour de l'Ascension"),
            (
                "2024-06-08",
                "Journée Nationale du Patriotisme et Commémoration de la Mort "
                "du Président Pierre Nkurunziza",
            ),
            ("2024-06-16", "Aid-El-Adha"),
            ("2024-06-17", "Aid-El-Adha (observé)"),
            ("2024-07-01", "Anniversaire de l'Indépendance"),
            ("2024-08-15", "Assomption"),
            (
                "2024-10-13",
                "Commémoration de l'Assassinat du Héros National, le Prince Louis Rwagasore",
            ),
            (
                "2024-10-14",
                "Commémoration de l'Assassinat du Héros National, "
                "le Prince Louis Rwagasore (observé)",
            ),
            ("2024-10-21", "Commémoration de l'Assassinat du Président Melchior Ndadaye"),
            ("2024-11-01", "Toussaint"),
            ("2024-12-25", "Noël"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Jour de l'an"),
            ("2025-02-05", "Fête de l'Unité"),
            ("2025-03-30", "Aid-El-Fithr"),
            ("2025-03-31", "Aid-El-Fithr (observé)"),
            ("2025-04-06", "Commémoration de l'Assassinat du Président Cyprien Ntaryamira"),
            (
                "2025-04-07",
                "Commémoration de l'Assassinat du Président Cyprien Ntaryamira (observé)",
            ),
            ("2025-05-01", "Fête Internationale du Travail"),
            ("2025-05-29", "Jour de l'Ascension"),
            ("2025-06-06", "Aid-El-Adha"),
            (
                "2025-06-08",
                "Journée Nationale du Patriotisme et Commémoration de la Mort "
                "du Président Pierre Nkurunziza",
            ),
            (
                "2025-06-09",
                "Journée Nationale du Patriotisme et Commémoration de la Mort "
                "du Président Pierre Nkurunziza (observé)",
            ),
            ("2025-07-01", "Anniversaire de l'Indépendance"),
            ("2025-08-15", "Assomption"),
            (
                "2025-10-13",
                "Commémoration de l'Assassinat du Héros National, le Prince Louis Rwagasore",
            ),
            ("2025-10-21", "Commémoration de l'Assassinat du Président Melchior Ndadaye"),
            ("2025-11-01", "Toussaint"),
            ("2025-12-25", "Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-02-05", "Unity Day"),
            ("2025-03-30", "Eid al-Fitr"),
            ("2025-03-31", "Eid al-Fitr (observed)"),
            ("2025-04-06", "Commemoration of the Assassination of President Cyprien Ntaryamira"),
            (
                "2025-04-07",
                "Commemoration of the Assassination of President Cyprien Ntaryamira (observed)",
            ),
            ("2025-05-01", "International Labor Day"),
            ("2025-05-29", "Ascension Day"),
            ("2025-06-06", "Eid al-Adha"),
            (
                "2025-06-08",
                "National Day of Patriotism and Commemoration of the Death of "
                "President Pierre Nkurunziza",
            ),
            (
                "2025-06-09",
                "National Day of Patriotism and Commemoration of the Death of "
                "President Pierre Nkurunziza (observed)",
            ),
            ("2025-07-01", "Independence Day"),
            ("2025-08-15", "Assumption Day"),
            (
                "2025-10-13",
                "Commemoration of the Assassination of National Hero, Prince Louis Rwagasore",
            ),
            ("2025-10-21", "Commemoration of the Assassination of President Melchior Ndadaye"),
            ("2025-11-01", "All Saints' Day"),
            ("2025-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.syrian_arab_republic import SyrianArabRepublic
from tests.common import CommonCountryTests


class TestSyrianArabRepublic(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(2004, 2050)
        super().setUpClass(SyrianArabRepublic, years=years)
        cls.no_estimated_holidays = SyrianArabRepublic(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        self.assertHolidayName(
            "عيد رأس السنة الميلادية", (f"{year}-01-01" for year in range(2004, 2050))
        )

    def test_revolution_day(self):
        self.assertHolidayName("الثورة السورية", (f"{year}-03-08" for year in range(2004, 2050)))

    def test_mothers_day(self):
        self.assertHolidayName("عيد الأم", (f"{year}-03-21" for year in range(2004, 2050)))

    def test_gregorian_easter_sunday(self):
        name = "عيد الفصح حسب التقويم الغربي"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(2004, 2050))

    def test_julian_easter_sunday(self):
        name = "عيد الفصح حسب التقويم الشرقي"
        self.assertHolidayName(
            name,
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
            "2025-04-20",
        )
        self.assertHolidayName(name, range(2004, 2050))

    def test_independence_day(self):
        self.assertHolidayName("عيد الاستقلال", (f"{year}-04-17" for year in range(2004, 2050)))

    def test_labor_day(self):
        self.assertHolidayName("عيد العمال", (f"{year}-05-01" for year in range(2004, 2050)))

    def test_martyrs_day(self):
        self.assertHolidayName("عيد الشهداء", (f"{year}-05-06" for year in range(2004, 2050)))

    def test_tishreen_liberation_war_day(self):
        self.assertHolidayName(
            "ذكرى حرب تشرين التحريرية", (f"{year}-10-06" for year in range(2004, 2050))
        )

    def test_liberation_day(self):
        name = "عيد التحرير"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(2025, 2050)))
        self.assertNoHolidayName(name, range(2004, 2025))

    def test_christmas_day(self):
        name = "عيد الميلاد"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(2024, 2050)))
        self.assertNoHolidayName(name, range(2004, 2024))

    def test_boxing_day(self):
        name = "يوم الصناديق"
        self.assertHolidayName(name, (f"{year}-12-26" for year in range(2024, 2050)))
        self.assertNoHolidayName(name, range(2004, 2024))

    def test_hijri_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))

    def test_mawlid(self):
        name = "عيد المولد النبوي الشريف"
        self.assertHolidayName(
            name,
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-02",
            "2022-05-03",
            "2022-05-04",
            "2023-04-21",
            "2023-04-22",
            "2023-04-23",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-31",
            "2025-04-01",
            "2025-04-02",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertHolidayName(
            name,
            "2020-07-30",
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-19",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-08",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2023-06-27",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-15",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-05",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2004, 2050))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "عيد رأس السنة الميلادية"),
            ("2024-03-08", "الثورة السورية"),
            ("2024-03-21", "عيد الأم"),
            ("2024-03-31", "عيد الفصح حسب التقويم الغربي"),
            ("2024-04-10", "عيد الفطر"),
            ("2024-04-11", "عيد الفطر"),
            ("2024-04-12", "عيد الفطر"),
            ("2024-04-17", "عيد الاستقلال"),
            ("2024-05-01", "عيد العمال"),
            ("2024-05-05", "عيد الفصح حسب التقويم الشرقي"),
            ("2024-05-06", "عيد الشهداء"),
            ("2024-06-15", "عيد الأضحى"),
            ("2024-06-16", "عيد الأضحى"),
            ("2024-06-17", "عيد الأضحى"),
            ("2024-06-18", "عيد الأضحى"),
            ("2024-07-07", "رأس السنة الهجرية"),
            ("2024-09-15", "عيد المولد النبوي الشريف"),
            ("2024-10-06", "ذكرى حرب تشرين التحريرية"),
            ("2024-12-25", "عيد الميلاد"),
            ("2024-12-26", "يوم الصناديق"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-08", "Revolution Day"),
            ("2024-03-21", "Mother's Day"),
            ("2024-03-31", "Gregorian Easter Sunday"),
            ("2024-04-10", "Eid al-Fitr"),
            ("2024-04-11", "Eid al-Fitr"),
            ("2024-04-12", "Eid al-Fitr"),
            ("2024-04-17", "Independence Day"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-05", "Julian Easter Sunday"),
            ("2024-05-06", "Martyrs' Day"),
            ("2024-06-15", "Eid al-Adha"),
            ("2024-06-16", "Eid al-Adha"),
            ("2024-06-17", "Eid al-Adha"),
            ("2024-06-18", "Eid al-Adha"),
            ("2024-07-07", "Islamic New Year"),
            ("2024-09-15", "Prophet's Birthday"),
            ("2024-10-06", "Tishreen Liberation War Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import WORKDAY
from holidays.countries.benin import Benin
from tests.common import CommonCountryTests


class TestBenin(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Benin)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Benin(categories=WORKDAY, years=range(self.start_year, 1990)))

    def test_new_years_day(self):
        self.assertHolidayName("Fête du Nouvel An", (f"{year}-01-01" for year in self.full_range))

    def test_vodoun_festival(self):
        name = "Fête annuelle des religions traditionnelles"
        self.assertHolidayName(name, (f"{year}-01-10" for year in range(1998, 2025)))
        self.assertHolidayName(
            name,
            "2025-01-09",
            "2025-01-10",
        )
        self.assertHolidayName(name, range(2025, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1998))

    def test_martyrs_day(self):
        name = "Journée des Martyrs"
        self.assertHolidayName(name, (f"{year}-01-16" for year in range(1980, 1991)))
        self.assertNoHolidayName(name, range(self.start_year, 1980), range(1991, self.end_year))

    def test_youth_day(self):
        name = "Journée de la Jeunesse Béninoise"
        self.assertHolidayName(name, (f"{year}-04-01" for year in range(1980, 1991)))
        self.assertNoHolidayName(name, range(self.start_year, 1980), range(1991, self.end_year))

    def test_easter_sunday(self):
        name = "Jour de Pâques"
        self.assertHolidayName(
            name,
            "1977-04-10",
            "1978-03-26",
            "1980-04-06",
            "1983-04-03",
            "1987-04-19",
            "1990-04-15",
        )
        self.assertHolidayName(name, range(self.start_year, 1991))
        self.assertNoHolidayName(name, range(1991, self.end_year))

    def test_easter_monday(self):
        name = "Lundi de Pâques"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1991, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1991))

    def test_labor_day(self):
        self.assertHolidayName("Fête du Travail", (f"{year}-05-01" for year in self.full_range))

    def test_ascension_day(self):
        name = "Jour de l'Ascension"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, range(1991, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1991))

    def test_whit_sunday(self):
        name = "Jour de Pentecôte"
        self.assertHolidayName(
            name,
            "1977-05-29",
            "1980-05-25",
            "1983-05-22",
            "1986-05-18",
            "1988-05-22",
            "1990-06-03",
        )
        self.assertHolidayName(name, range(self.start_year, 1991))
        self.assertNoHolidayName(name, range(1991, self.end_year))

    def test_whit_monday(self):
        name = "Lundi de Pentecôte"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, range(1991, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 1991))

    def test_national_day(self):
        self.assertHolidayName(
            "Fête Nationale",
            (f"{year}-11-30" for year in range(self.start_year, 1991)),
            (f"{year}-08-01" for year in range(1991, self.end_year)),
        )

    def test_assumption_day(self):
        name = "Jour de l'Assomption"
        self.assertHolidayName(name, (f"{year}-08-15" for year in range(1990, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1990))

    def test_day_of_the_popular_armed_forces(self):
        name = "Fête des Forces Armées Populaires du Bénin"
        self.assertHolidayName(name, (f"{year}-10-26" for year in range(self.start_year, 1990)))
        self.assertNoHolidayName(name, range(1990, self.end_year))

    def test_all_saints_day(self):
        name = "La Toussaint"
        self.assertHolidayName(name, (f"{year}-11-01" for year in range(1990, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1990))

    def test_christmas_day(self):
        self.assertHolidayName("Jour de Noël", (f"{year}-12-25" for year in self.full_range))

    def test_production_day(self):
        name = "Fête de la Production"
        self.assertHolidayName(name, (f"{year}-12-31" for year in range(self.start_year, 1990)))
        self.assertNoHolidayName(name, range(1990, self.end_year))

    def test_prophets_birthday(self):
        name = "Journée Maouloud"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1990, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1990))

    def test_eid_al_fitr(self):
        name = "Jour du Ramadan"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-21",
            "2024-04-10",
            "2025-03-30",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_adha(self):
        name = "Jour de la Tabaski"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-28",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_remembrance_day(self):
        name = "Journée de Souvenir"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-01-16" for year in range(1990, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1990))

    def test_peoples_sovereignty_day(self):
        name = "Journée de la Souveraineté de Peuple"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-02-28" for year in range(1990, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1990))

    def test_womens_day(self):
        name = "Journée de la Femme"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-08" for year in range(1990, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 1990))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Fête du Nouvel An"),
            ("2022-01-10", "Fête annuelle des religions traditionnelles"),
            ("2022-01-16", "Journée de Souvenir"),
            ("2022-02-28", "Journée de la Souveraineté de Peuple"),
            ("2022-03-08", "Journée de la Femme"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-02", "Jour du Ramadan (estimé)"),
            ("2022-05-26", "Jour de l'Ascension"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-07-09", "Jour de la Tabaski (estimé)"),
            ("2022-08-01", "Fête Nationale"),
            ("2022-08-15", "Jour de l'Assomption"),
            ("2022-10-08", "Journée Maouloud (estimé)"),
            ("2022-11-01", "La Toussaint"),
            ("2022-12-25", "Jour de Noël"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-10", "Vodoun Festival"),
            ("2022-01-16", "Remembrance Day"),
            ("2022-02-28", "People's Sovereignty Day"),
            ("2022-03-08", "Women's Day"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr (estimated)"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-07-09", "Eid al-Adha (estimated)"),
            ("2022-08-01", "National Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-10-08", "Prophet's Birthday (estimated)"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest
from datetime import date

from holidays import calendars
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.calendars.thai import KHMER_CALENDAR


class TestThaiLunisolarCalendar(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()
        self.calendar = calendars._ThaiLunisolar()

    def test_check_calendar(self):
        self.assertRaises(ValueError, lambda: calendars._ThaiLunisolar("INVALID_CALENDAR"))

    def test_asarnha_bucha_date(self):
        # THAI_CALENDAR
        asarnha_bucha_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2010: date(2010, JUL, 26),
            2011: date(2011, JUL, 15),
            2012: date(2012, AUG, 2),
            2013: date(2013, JUL, 22),
            2014: date(2014, JUL, 11),
            2015: date(2015, JUL, 30),
            2016: date(2016, JUL, 19),
            2017: date(2017, JUL, 8),
            2018: date(2018, JUL, 27),
            2019: date(2019, JUL, 16),
            2020: date(2020, JUL, 5),
            2021: date(2021, JUL, 24),
            2022: date(2022, JUL, 13),
            2023: date(2023, AUG, 1),
            2024: date(2024, JUL, 20),
            2025: date(2025, JUL, 10),
            2026: date(2026, JUL, 29),
            2027: date(2027, JUL, 18),
            2028: date(2028, JUL, 6),
            2029: date(2029, JUL, 25),
            2030: date(2030, JUL, 14),
            2040: date(2040, JUL, 23),
            2050: date(2050, AUG, 2),
            2060: date(2060, JUL, 13),
            2070: date(2070, JUL, 22),
            2080: date(2080, JUL, 31),
            2090: date(2090, JUL, 11),
            2100: date(2100, JUL, 21),
            2110: date(2110, JUL, 1),
            2120: date(2120, JUL, 10),
            2130: date(2130, JUL, 20),
            2140: date(2140, JUN, 29),
            2150: date(2150, JUL, 9),
        }
        for year in asarnha_bucha_year_date:
            self.assertEqual(asarnha_bucha_year_date[year], self.calendar.asarnha_bucha_date(year))

    def test_atthami_bucha_date(self):
        # THAI_CALENDAR
        atthami_bucha_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, MAY, 23),
            2023: date(2023, JUN, 11),
            2024: date(2024, MAY, 30),
            2025: date(2025, MAY, 19),
            2030: date(2030, MAY, 24),
            2040: date(2040, JUN, 2),
            2050: date(2050, JUN, 12),
            2060: date(2060, MAY, 23),
            2070: date(2070, JUN, 1),
            2080: date(2080, JUN, 10),
            2090: date(2090, MAY, 21),
            2100: date(2100, MAY, 31),
            2110: date(2110, MAY, 11),
            2120: date(2120, MAY, 20),
            2130: date(2130, MAY, 30),
            2140: date(2140, MAY, 9),
            2150: date(2150, MAY, 19),
        }
        for year in atthami_bucha_year_date:
            self.assertEqual(atthami_bucha_year_date[year], self.calendar.atthami_bucha_date(year))
        # KHMER_CALENDAR
        athami_bochea_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, MAY, 23),
            2023: date(2023, MAY, 12),
            2024: date(2024, MAY, 30),
            2025: date(2025, MAY, 19),
        }
        for year in athami_bochea_year_date:
            self.assertEqual(
                athami_bochea_year_date[year],
                self.calendar.atthami_bucha_date(year, KHMER_CALENDAR),
            )

    def test_boun_haw_khao_padapdin(self):
        boun_haw_khao_padapdin_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, AUG, 26),
            2023: date(2023, SEP, 14),
            2024: date(2024, SEP, 2),
            2025: date(2025, AUG, 23),
        }
        for year in boun_haw_khao_padapdin_year_date:
            self.assertEqual(
                boun_haw_khao_padapdin_year_date[year],
                self.calendar.boun_haw_khao_padapdin_date(year),
            )

    def test_boun_haw_khao_salark(self):
        boun_haw_khao_salark_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, SEP, 10),
            2023: date(2023, SEP, 29),
            2024: date(2024, SEP, 17),
            2025: date(2025, SEP, 7),
        }
        for year in boun_haw_khao_salark_year_date:
            self.assertEqual(
                boun_haw_khao_salark_year_date[year], self.calendar.boun_haw_khao_salark_date(year)
            )

    def test_boun_suang_heua_date(self):
        boun_suang_heua_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, OCT, 11),
            2023: date(2023, OCT, 30),
            2024: date(2024, OCT, 18),
            2025: date(2025, OCT, 8),
        }
        for year in boun_suang_heua_year_date:
            self.assertEqual(
                boun_suang_heua_year_date[year], self.calendar.boun_suang_heua_date(year)
            )

    def test_buddhist_sabbath_dates(self):
        expected_buddhist_sabbath_dates = {
            self.calendar.START_YEAR - 1: set(),
            self.calendar.END_YEAR + 1: set(),
            # Athikamat (Extra-Month Year).
            2023: {
                date(2023, JAN, 6),  # 15 Waxing (Month 2 Started in Previous Gregorian Year).
                date(2023, JAN, 14),  # 8 Waning.
                date(2023, JAN, 21),  # 15 Waning.
                date(2023, JAN, 29),  # 8 Waxing (Month 3).
                date(2023, FEB, 5),  # 15 Waxing.
                date(2023, FEB, 13),  # 8 Waning.
                date(2023, FEB, 19),  # 14 Waning.
                date(2023, FEB, 27),  # 8 Waxing (Month 4).
                date(2023, MAR, 6),  # 15 Waxing.
                date(2023, MAR, 14),  # 8 Waning.
                date(2023, MAR, 21),  # 15 Waning.
                date(2023, MAR, 29),  # 8 Waxing (Month 5).
                date(2023, APR, 5),  # 15 Waxing.
                date(2023, APR, 13),  # 8 Waning.
                date(2023, APR, 19),  # 14 Waning.
                date(2023, APR, 27),  # 8 Waxing (Month 6).
                date(2023, MAY, 4),  # 15 Waxing.
                date(2023, MAY, 12),  # 8 Waning.
                date(2023, MAY, 19),  # 15 Waning.
                date(2023, MAY, 27),  # 8 Waxing (Month 7).
                date(2023, JUN, 3),  # 15 Waxing.
                date(2023, JUN, 11),  # 8 Waning.
                date(2023, JUN, 17),  # 14 Waning.
                date(2023, JUN, 25),  # 8 Waxing (Month 8).
                date(2023, JUL, 2),  # 15 Waxing.
                date(2023, JUL, 10),  # 8 Waning.
                date(2023, JUL, 17),  # 14 Waning.
                date(2023, JUL, 25),  # 8 Waxing (Month 8.8).
                date(2023, AUG, 1),  # 15 Waxing.
                date(2023, AUG, 9),  # 8 Waning.
                date(2023, AUG, 16),  # 15 Waning.
                date(2023, AUG, 24),  # 8 Waxing (Month 9).
                date(2023, AUG, 31),  # 15 Waxing.
                date(2023, SEP, 8),  # 8 Waning.
                date(2023, SEP, 14),  # 14 Waning.
                date(2023, SEP, 22),  # 8 Waxing (Month 10).
                date(2023, SEP, 29),  # 15 Waxing.
                date(2023, OCT, 7),  # 8 Waning.
                date(2023, OCT, 14),  # 15 Waning.
                date(2023, OCT, 22),  # 8 Waxing (Month 11).
                date(2023, OCT, 29),  # 15 Waxing.
                date(2023, NOV, 6),  # 8 Waning.
                date(2023, NOV, 12),  # 14 Waning.
                date(2023, NOV, 20),  # 8 Waxing (Month 12).
                date(2023, NOV, 27),  # 15 Waxing.
                date(2023, DEC, 5),  # 8 Waning.
                date(2023, DEC, 12),  # 15 Waning.
                date(2023, DEC, 20),  # 8 Waxing (Month 1 Next Year).
                date(2023, DEC, 27),  # 15 Waxing.
            },
            # Pakatimat (Normal Year).
            2024: {
                date(2024, JAN, 4),  # 8 Waning (Month 1 Started in Previous Gregorian Year).
                date(2024, JAN, 10),  # 14 Waning.
                date(2024, JAN, 18),  # 8 Waxing (Month 2).
                date(2024, JAN, 25),  # 15 Waxing.
                date(2024, FEB, 2),  # 8 Waning.
                date(2024, FEB, 9),  # 15 Waning.
                date(2024, FEB, 17),  # 8 Waxing (Month 3).
                date(2024, FEB, 24),  # 15 Waxing.
                date(2024, MAR, 3),  # 8 Waning.
                date(2024, MAR, 9),  # 14 Waning.
                date(2024, MAR, 17),  # 8 Waxing (Month 4).
                date(2024, MAR, 24),  # 15 Waxing.
                date(2024, APR, 1),  # 8 Waning.
                date(2024, APR, 8),  # 15 Waning.
                date(2024, APR, 16),  # 8 Waxing (Month 5).
                date(2024, APR, 23),  # 15 Waxing.
                date(2024, MAY, 1),  # 8 Waning.
                date(2024, MAY, 7),  # 14 Waning.
                date(2024, MAY, 15),  # 8 Waxing (Month 6).
                date(2024, MAY, 22),  # 15 Waxing.
                date(2024, MAY, 30),  # 8 Waning.
                date(2024, JUN, 6),  # 15 Waning.
                date(2024, JUN, 14),  # 8 Waxing (Month 7).
                date(2024, JUN, 21),  # 15 Waxing.
                date(2024, JUN, 29),  # 8 Waning.
                date(2024, JUL, 5),  # 14 Waning.
                date(2024, JUL, 13),  # 8 Waxing (Month 8).
                date(2024, JUL, 20),  # 15 Waxing.
                date(2024, JUL, 28),  # 8 Waning.
                date(2024, AUG, 4),  # 15 Waning.
                date(2024, AUG, 12),  # 8 Waxing (Month 9).
                date(2024, AUG, 19),  # 15 Waxing.
                date(2024, AUG, 27),  # 8 Waning.
                date(2024, SEP, 2),  # 14 Waning.
                date(2024, SEP, 10),  # 8 Waxing (Month 10).
                date(2024, SEP, 17),  # 15 Waxing.
                date(2024, SEP, 25),  # 8 Waning.
                date(2024, OCT, 2),  # 15 Waning.
                date(2024, OCT, 10),  # 8 Waxing (Month 11).
                date(2024, OCT, 17),  # 15 Waxing.
                date(2024, OCT, 25),  # 8 Waning.
                date(2024, OCT, 31),  # 15 Waning.
                date(2024, NOV, 8),  # 8 Waxing (Month 12).
                date(2024, NOV, 15),  # 15 Waxing.
                date(2024, NOV, 23),  # 8 Waning.
                date(2024, NOV, 30),  # 15 Waning.
                date(2024, DEC, 8),  # 8 Waxing (Month 1 Next Year).
                date(2024, DEC, 15),  # 15 Waxing.
                date(2024, DEC, 23),  # 8 Waning.
                date(2024, DEC, 29),  # 14 Waning.
            },
            # Athikawan (Extra-Day Year).
            2025: {
                date(2025, JAN, 6),  # 8 Waxing (Month 2).
                date(2025, JAN, 13),  # 15 Waxing.
                date(2025, JAN, 21),  # 8 Waning.
                date(2025, JAN, 28),  # 15 Waning.
                date(2025, FEB, 5),  # 8 Waxing (Month 3).
                date(2025, FEB, 12),  # 15 Waxing.
                date(2025, FEB, 20),  # 8 Waning.
                date(2025, FEB, 26),  # 14 Waning.
                date(2025, MAR, 6),  # 8 Waxing (Month 4).
                date(2025, MAR, 13),  # 15 Waxing.
                date(2025, MAR, 21),  # 8 Waning.
                date(2025, MAR, 28),  # 15 Waning.
                date(2025, APR, 5),  # 8 Waxing (Month 5).
                date(2025, APR, 12),  # 15 Waxing.
                date(2025, APR, 20),  # 8 Waning.
                date(2025, APR, 26),  # 14 Waning.
                date(2025, MAY, 4),  # 8 Waxing (Month 6).
                date(2025, MAY, 11),  # 15 Waxing.
                date(2025, MAY, 19),  # 8 Waning.
                date(2025, MAY, 26),  # 15 Waning.
                date(2025, JUN, 3),  # 8 Waxing (Month 7).
                date(2025, JUN, 10),  # 15 Waxing.
                date(2025, JUN, 18),  # 8 Waning.
                date(2025, JUN, 25),  # 15 Waning.
                date(2025, JUL, 3),  # 8 Waxing (Month 8).
                date(2025, JUL, 10),  # 15 Waxing.
                date(2025, JUL, 18),  # 8 Waning.
                date(2025, JUL, 25),  # 15 Waning.
                date(2025, AUG, 2),  # 8 Waxing (Month 9).
                date(2025, AUG, 9),  # 15 Waxing.
                date(2025, AUG, 17),  # 8 Waning.
                date(2025, AUG, 23),  # 14 Waning.
                date(2025, AUG, 31),  # 8 Waxing (Month 10).
                date(2025, SEP, 7),  # 15 Waxing.
                date(2025, SEP, 15),  # 8 Waning.
                date(2025, SEP, 22),  # 15 Waning.
                date(2025, SEP, 30),  # 8 Waxing (Month 11).
                date(2025, OCT, 7),  # 15 Waxing.
                date(2025, OCT, 15),  # 8 Waning.
                date(2025, OCT, 21),  # 15 Waning.
                date(2025, OCT, 29),  # 8 Waxing (Month 12).
                date(2025, NOV, 5),  # 15 Waxing.
                date(2025, NOV, 13),  # 8 Waning.
                date(2025, NOV, 20),  # 15 Waning.
                date(2025, NOV, 28),  # 8 Waxing (Month 1 Next Year).
                date(2025, DEC, 5),  # 15 Waxing.
                date(2025, DEC, 13),  # 8 Waning.
                date(2025, DEC, 19),  # 14 Waning.
                date(2025, DEC, 27),  # 8 Waxing (Month 2 Next Year).
            },
        }
        for year, expected_dates in expected_buddhist_sabbath_dates.items():
            result = self.calendar.buddhist_sabbath_dates(year)
            self.assertEqual(result, expected_dates)

    def test_khao_phansa_date(self):
        # THAI_CALENDAR
        khao_phansa_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2010: date(2010, JUL, 27),
            2011: date(2011, JUL, 16),
            2012: date(2012, AUG, 3),
            2013: date(2013, JUL, 23),
            2014: date(2014, JUL, 12),
            2015: date(2015, JUL, 31),
            2016: date(2016, JUL, 20),
            2017: date(2017, JUL, 9),
            2018: date(2018, JUL, 28),
            2019: date(2019, JUL, 17),
            2020: date(2020, JUL, 6),
            2021: date(2021, JUL, 25),
            2022: date(2022, JUL, 14),
            2023: date(2023, AUG, 2),
            2024: date(2024, JUL, 21),
            2025: date(2025, JUL, 11),
            2026: date(2026, JUL, 30),
            2027: date(2027, JUL, 19),
            2028: date(2028, JUL, 7),
            2029: date(2029, JUL, 26),
            2030: date(2030, JUL, 15),
            2040: date(2040, JUL, 24),
            2050: date(2050, AUG, 3),
            2060: date(2060, JUL, 14),
            2070: date(2070, JUL, 23),
            2080: date(2080, AUG, 1),
            2090: date(2090, JUL, 12),
            2100: date(2100, JUL, 22),
            2110: date(2110, JUL, 2),
            2120: date(2120, JUL, 11),
            2130: date(2130, JUL, 21),
            2140: date(2140, JUN, 30),
            2150: date(2150, JUL, 10),
        }
        for year in khao_phansa_year_date:
            self.assertEqual(khao_phansa_year_date[year], self.calendar.khao_phansa_date(year))

    def test_loy_krathong_date(self):
        loy_krathong_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2015: date(2015, NOV, 25),
            2016: date(2016, NOV, 14),
            2017: date(2017, NOV, 3),
            2018: date(2018, NOV, 22),
            2019: date(2019, NOV, 11),
            2020: date(2020, OCT, 31),
            2021: date(2021, NOV, 19),
            2022: date(2022, NOV, 8),
            2023: date(2023, NOV, 27),
            2024: date(2024, NOV, 15),
            2025: date(2025, NOV, 5),
            2030: date(2030, NOV, 9),
            2040: date(2040, NOV, 18),
            2050: date(2050, NOV, 28),
            2060: date(2060, NOV, 8),
            2070: date(2070, NOV, 17),
            2080: date(2080, NOV, 26),
            2090: date(2090, NOV, 6),
            2100: date(2100, NOV, 16),
            2110: date(2110, OCT, 27),
            2120: date(2120, NOV, 5),
            2130: date(2130, NOV, 15),
            2140: date(2140, OCT, 25),
            2150: date(2150, NOV, 4),
        }
        for year in loy_krathong_year_date:
            self.assertEqual(loy_krathong_year_date[year], self.calendar.loy_krathong_date(year))

    def test_makha_bucha_date(self):
        # THAI_CALENDAR
        makha_bucha_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2010: date(2010, FEB, 28),
            2011: date(2011, FEB, 18),
            2012: date(2012, MAR, 7),
            2013: date(2013, FEB, 25),
            2014: date(2014, FEB, 14),
            2015: date(2015, MAR, 4),
            2016: date(2016, FEB, 22),
            2017: date(2017, FEB, 11),
            2018: date(2018, MAR, 1),
            2019: date(2019, FEB, 19),
            2020: date(2020, FEB, 8),
            2021: date(2021, FEB, 26),
            2022: date(2022, FEB, 16),
            2023: date(2023, MAR, 6),
            2024: date(2024, FEB, 24),
            2025: date(2025, FEB, 12),
            2026: date(2026, MAR, 3),
            2027: date(2027, FEB, 21),
            2028: date(2028, FEB, 10),
            2029: date(2029, FEB, 27),
            2030: date(2030, FEB, 17),
            2040: date(2040, FEB, 26),
            2050: date(2050, MAR, 7),
            2060: date(2060, FEB, 17),
            2070: date(2070, FEB, 25),
            2080: date(2080, MAR, 5),
            2090: date(2090, FEB, 14),
            2100: date(2100, FEB, 24),
            2110: date(2110, FEB, 4),
            2120: date(2120, FEB, 14),
            2130: date(2130, FEB, 22),
            2140: date(2140, FEB, 3),
            2150: date(2150, FEB, 12),
        }
        for year in makha_bucha_year_date:
            self.assertEqual(makha_bucha_year_date[year], self.calendar.makha_bucha_date(year))
        # KHMER_CALENDAR
        meak_bochea_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2015: date(2015, FEB, 3),
            2016: date(2016, FEB, 22),
            2017: date(2017, FEB, 11),
            2018: date(2018, JAN, 31),
            2019: date(2019, FEB, 19),
            2022: date(2022, FEB, 16),
            2023: date(2023, FEB, 5),
            2024: date(2024, FEB, 24),
            2025: date(2025, FEB, 12),
        }
        for year in meak_bochea_year_date:
            self.assertEqual(
                meak_bochea_year_date[year], self.calendar.makha_bucha_date(year, KHMER_CALENDAR)
            )

    def test_ok_phansa_date(self):
        ok_phansa_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2022: date(2022, OCT, 10),
            2023: date(2023, OCT, 29),
            2024: date(2024, OCT, 17),
            2025: date(2025, OCT, 7),
            2030: date(2030, OCT, 11),
            2040: date(2040, OCT, 20),
            2050: date(2050, OCT, 30),
            2060: date(2060, OCT, 10),
            2070: date(2070, OCT, 19),
            2080: date(2080, OCT, 28),
            2090: date(2090, OCT, 8),
            2100: date(2100, OCT, 18),
            2110: date(2110, SEP, 28),
            2120: date(2120, OCT, 7),
            2130: date(2130, OCT, 17),
            2140: date(2140, SEP, 26),
            2150: date(2150, OCT, 6),
        }
        for year in ok_phansa_year_date:
            self.assertEqual(ok_phansa_year_date[year], self.calendar.ok_phansa_date(year))

    def test_pchum_ben_date(self):
        pchum_ben_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2015: date(2015, OCT, 12),
            2016: date(2016, OCT, 1),
            2017: date(2017, SEP, 20),
            2018: date(2018, OCT, 9),
            2019: date(2019, SEP, 28),
            2020: date(2020, SEP, 17),
            2021: date(2021, OCT, 6),
            2022: date(2022, SEP, 25),
            2023: date(2023, OCT, 14),
            2024: date(2024, OCT, 2),
            2025: date(2025, SEP, 22),
        }
        for year in pchum_ben_year_date:
            self.assertEqual(pchum_ben_year_date[year], self.calendar.pchum_ben_date(year))

    def test_preah_neangkoal_date(self):
        preah_neangkoal_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2015: date(2015, MAY, 6),
            2016: date(2016, MAY, 24),
            2017: date(2017, MAY, 14),
            2018: date(2018, MAY, 3),
            2019: date(2019, MAY, 22),
            2020: date(2020, MAY, 10),
            2021: date(2021, APR, 30),
            2022: date(2022, MAY, 19),
            2023: date(2023, MAY, 8),
            2024: date(2024, MAY, 26),
            2025: date(2025, MAY, 15),
        }
        for year in preah_neangkoal_year_date:
            self.assertEqual(
                preah_neangkoal_year_date[year], self.calendar.preah_neangkoal_date(year)
            )

    def test_visakha_bucha_date(self):
        # THAI_CALENDAR
        visakha_bucha_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2010: date(2010, MAY, 28),
            2011: date(2011, MAY, 17),
            2012: date(2012, JUN, 4),
            2013: date(2013, MAY, 24),
            2014: date(2014, MAY, 13),
            2015: date(2015, JUN, 1),
            2016: date(2016, MAY, 20),
            2017: date(2017, MAY, 10),
            2018: date(2018, MAY, 29),
            2019: date(2019, MAY, 18),
            2020: date(2020, MAY, 6),
            2021: date(2021, MAY, 26),
            2022: date(2022, MAY, 15),
            2023: date(2023, JUN, 3),
            2024: date(2024, MAY, 22),
            2025: date(2025, MAY, 11),
            2026: date(2026, MAY, 31),
            2027: date(2027, MAY, 20),
            2028: date(2028, MAY, 8),
            2029: date(2029, MAY, 27),
            2030: date(2030, MAY, 16),
            2040: date(2040, MAY, 25),
            2050: date(2050, JUN, 4),
            2060: date(2060, MAY, 15),
            2070: date(2070, MAY, 24),
            2080: date(2080, JUN, 2),
            2090: date(2090, MAY, 13),
            2100: date(2100, MAY, 23),
            2110: date(2110, MAY, 3),
            2120: date(2120, MAY, 12),
            2130: date(2130, MAY, 22),
            2140: date(2140, MAY, 1),
            2150: date(2150, MAY, 11),
        }
        for year in visakha_bucha_year_date:
            self.assertEqual(visakha_bucha_year_date[year], self.calendar.visakha_bucha_date(year))
        # KHMER_CALENDAR
        visaka_bochea_year_date = {
            self.calendar.START_YEAR - 1: None,
            self.calendar.END_YEAR + 1: None,
            2015: date(2015, MAY, 2),
            2016: date(2016, MAY, 20),
            2017: date(2017, MAY, 10),
            2018: date(2018, APR, 29),
            2019: date(2019, MAY, 18),
            2020: date(2020, MAY, 6),
            2021: date(2021, APR, 26),
            2022: date(2022, MAY, 15),
            2023: date(2023, MAY, 4),
            2024: date(2024, MAY, 22),
            2025: date(2025, MAY, 11),
        }
        for year in visaka_bochea_year_date:
            self.assertEqual(
                visaka_bochea_year_date[year],
                self.calendar.visakha_bucha_date(year, KHMER_CALENDAR),
            )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.suriname import Suriname
from tests.common import CommonCountryTests


class TestSuriname(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        years = range(1972, 2050)
        super().setUpClass(Suriname, years=years)
        cls.no_estimated_holidays = Suriname(years=years, islamic_show_estimated=False)

    def test_new_years_day(self):
        self.assertHolidayName("Nieuwjaarsdag", (f"{year}-01-01" for year in range(1972, 2050)))

    def test_day_of_liberation_and_renewal(self):
        name = "Dag van Bevrijding en Vernieuwing"
        self.assertHolidayName(
            name, (f"{year}-02-25" for year in (*range(1981, 1993), *range(2012, 2021)))
        )
        self.assertNoHolidayName(name, range(1993, 2012), range(2021, 2050))

    def test_holi(self):
        name = "Holi-Phagwa"
        self.assertHolidayName(
            name,
            "2019-03-21",
            "2020-03-09",
            "2021-03-28",
            "2022-03-18",
            "2023-03-07",
            "2024-03-25",
            "2025-03-14",
        )
        self.assertHolidayName(name, range(2015, 2031))

    def test_good_friday(self):
        name = "Goede Vrijdag"
        self.assertHolidayName(
            name,
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, range(1972, 2050))

    def test_easter_monday(self):
        name = "Tweede Paasdag"
        self.assertHolidayName(
            name,
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, range(1972, 2050))

    def test_queens_birthday(self):
        name = "Verjaardag van H.M. de Koningin"
        self.assertHolidayName(name, (f"{year}-04-30" for year in range(1972, 1976)))
        self.assertNoHolidayName(name, range(1976, 2050))

    def test_labor_day(self):
        self.assertHolidayName(
            "Dag van de Arbeid", (f"{year}-05-01" for year in range(1972, 2050))
        )

    def test_day_of_freedoms(self):
        name_1 = "Dag der Vrijheden"
        name_2 = "Keti Koti Dey"
        self.assertHolidayName(
            name_1, (f"{year}-07-01" for year in (*range(1972, 2008), *range(2025, 2050)))
        )
        self.assertHolidayName(name_2, (f"{year}-07-01" for year in range(2008, 2025)))
        self.assertNoHolidayName(name_1, range(2008, 2025))
        self.assertNoHolidayName(name_2, range(1972, 2008), range(2025, 2050))

    def test_indigenous_people_day(self):
        name = "Dag der Inheemsen"
        self.assertHolidayName(name, (f"{year}-08-09" for year in range(2007, 2050)))
        self.assertNoHolidayName(name, range(1972, 2007))

    def test_day_of_the_maroons(self):
        name = "Dag der Marrons"
        self.assertHolidayName(name, (f"{year}-10-10" for year in range(2012, 2050)))
        self.assertNoHolidayName(name, range(1972, 2012))

    def test_diwali(self):
        name = "Divali"
        self.assertHolidayName(
            name,
            "2020-11-14",
            "2021-11-04",
            "2022-10-24",
            "2023-11-12",
            "2024-10-31",
            "2025-10-20",
        )
        self.assertHolidayName(name, range(2012, 2050))
        self.assertNoHolidayName(name, range(1972, 2012))

    def test_republic_day(self):
        name_old = "Dag van de Republiek"
        name_new = "Onafhankelijkheidsdag"
        self.assertHolidayName(name_old, (f"{year}-11-25" for year in range(1976, 2008)))
        self.assertHolidayName(name_new, (f"{year}-11-25" for year in range(2008, 2050)))
        self.assertNoHolidayName(name_old, range(1972, 1976))
        self.assertNoHolidayName(name_new, range(1972, 2008))

    def test_christmas_day(self):
        self.assertHolidayName("Eerste Kerstdag", (f"{year}-12-25" for year in range(1972, 2050)))

    def test_second_day_of_christmas(self):
        self.assertHolidayName("Tweede Kerstdag", (f"{year}-12-26" for year in range(1972, 2050)))

    def test_eid_al_fitr(self):
        name = "Ied-Ul-Fitre"
        self.assertHolidayName(
            name,
            "2019-06-05",
            "2020-05-24",
            "2021-05-13",
            "2022-05-02",
            "2023-04-22",
            "2024-04-10",
            "2025-03-31",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(1972, 2050))

    def test_eid_al_adha(self):
        name = "Ied-Ul-Adha"
        self.assertHolidayName(
            name,
            "2019-08-12",
            "2020-07-31",
            "2021-07-20",
            "2022-07-09",
            "2023-06-29",
            "2024-06-16",
            "2025-06-07",
        )
        self.assertHolidayName(name, self.no_estimated_holidays, range(2012, 2050))
        self.assertNoHolidayName(name, self.no_estimated_holidays, range(1972, 2012))

    def test_chinese_new_year(self):
        name = "Chinees Nieuwjaar"
        self.assertHolidayName(
            name,
            "2022-02-01",
            "2023-01-22",
            "2024-02-10",
            "2025-01-29",
        )
        self.assertHolidayName(name, range(2022, 2050))
        self.assertNoHolidayName(name, range(1972, 2022))

    def test_2024(self):
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "Nieuwjaarsdag"),
            ("2024-02-10", "Chinees Nieuwjaar"),
            ("2024-03-25", "Holi-Phagwa"),
            ("2024-03-29", "Goede Vrijdag"),
            ("2024-04-01", "Tweede Paasdag"),
            ("2024-04-10", "Ied-Ul-Fitre"),
            ("2024-05-01", "Dag van de Arbeid"),
            ("2024-06-16", "Ied-Ul-Adha"),
            ("2024-07-01", "Keti Koti Dey"),
            ("2024-08-09", "Dag der Inheemsen"),
            ("2024-10-10", "Dag der Marrons"),
            ("2024-10-31", "Divali"),
            ("2024-11-25", "Onafhankelijkheidsdag"),
            ("2024-12-25", "Eerste Kerstdag"),
            ("2024-12-26", "Tweede Kerstdag"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2025-01-01", "Nieuwjaarsdag"),
            ("2025-01-29", "Chinees Nieuwjaar"),
            ("2025-03-14", "Holi-Phagwa"),
            ("2025-03-31", "Ied-Ul-Fitre"),
            ("2025-04-18", "Goede Vrijdag"),
            ("2025-04-21", "Tweede Paasdag"),
            ("2025-05-01", "Dag van de Arbeid"),
            ("2025-06-07", "Ied-Ul-Adha"),
            ("2025-07-01", "Dag der Vrijheden"),
            ("2025-08-09", "Dag der Inheemsen"),
            ("2025-10-10", "Dag der Marrons"),
            ("2025-10-20", "Divali"),
            ("2025-11-25", "Onafhankelijkheidsdag"),
            ("2025-12-25", "Eerste Kerstdag"),
            ("2025-12-26", "Tweede Kerstdag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2025-01-01", "New Year's Day"),
            ("2025-01-29", "Chinese New Year"),
            ("2025-03-14", "Holi"),
            ("2025-03-31", "Eid al-Fitr"),
            ("2025-04-18", "Good Friday"),
            ("2025-04-21", "Easter Monday"),
            ("2025-05-01", "Labor Day"),
            ("2025-06-07", "Eid al-Adha"),
            ("2025-07-01", "Day of Freedoms"),
            ("2025-08-09", "Indigenous People Day"),
            ("2025-10-10", "Day of the Maroons"),
            ("2025-10-20", "Diwali"),
            ("2025-11-25", "Independence Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Second Day of Christmas"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JUN
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class TurksAndCaicosIslands(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Turks and Caicos Islands holidays.

    References:
        * [Wikipedia](https://en.wikipedia.org/wiki/Public_holidays_in_the_Turks_and_Caicos_Islands)
        * [Public Holidays Ordinance, rev. 2014](https://web.archive.org/web/20250210082429/https://gov.tc/agc/component/edocman/21-02-public-holidays-ordinance-2/viewdocument/599?Itemid=)
        * [Ordinance 5 of 2020](https://web.archive.org/web/20250429025117/https://www.gov.tc/agc/component/edocman/05-of-2020-public-holidays-amendment-ordinance/viewdocument/1419?Itemid=)
        * [Public Holidays Ordinance, rev. 2021](https://web.archive.org/web/20250429025602/https://www.gov.tc/agc/component/edocman/21-02-public-holidays-ordinance-2/viewdocument/2027?Itemid=)
        * [2017](https://web.archive.org/web/20250608202444/https://www.facebook.com/photo/?fbid=1137860329642985&set=a.349345645161128)
        * [2018](https://web.archive.org/web/20180126185141/https://gov.tc/pressoffice/999-listing-of-special-days-public-holidays-for-2018)
        * [2019](https://web.archive.org/web/20250608202446/https://www.facebook.com/photo/?fbid=2514587681970236&set=a.349345645161128)
        * [2020](https://web.archive.org/web/20250608202458/https://www.facebook.com/pressofficetcig/photos/a.349345645161128/2572825079479829/?type=3)
        * [2021](https://www.facebook.com/pressofficetcig/photos/a.349345645161128/3511371698958491/?type=3)
        * [2022](https://www.facebook.com/pressofficetcig/photos/a.349345645161128/4515540038541647/?type=3)
        * [2023](https://web.archive.org/web/20250608202452/https://www.facebook.com/photo/?fbid=557741379734203&set=a.363028765872133)
        * [2025](https://web.archive.org/web/20250608202433/https://www.facebook.com/photo/?fbid=1019082573584944&set=a.353200576839817)
        * [Destination TCI](https://web.archive.org/web/20250429024708/https://destinationtci.tc/turks-and-caicos-islands-public-holidays/)
        * [Time and Date](https://web.archive.org/web/20250429024853/https://www.timeanddate.com/holidays/turks-and-caicos-islands/)
        * [Constitution Day](https://web.archive.org/web/20251230163946/https://www.visittci.com/news/turks-and-caicos-public-holiday-changes-2026)
    """

    country = "TC"
    default_language = "en_TC"
    # %s (observed).
    observed_label = "%s (observed)"
    # Separated from Jamaica in 1962.
    start_year = 1963
    supported_languages = ("en_TC", "en_US")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        # Public Holidays Act 1980 established weekend-to-Monday rule.
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 1980)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Commonwealth Day.
        self._add_holiday_2nd_mon_of_mar(tr("Commonwealth Day"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        self._add_holiday_last_mon_of_may(
            # JAGS McCartney Day.
            tr("JAGS McCartney Day")
            if self._year >= 2020
            # National Heroes Day.
            else tr("National Heroes Day")
        )

        name = (
            # King's Birthday.
            tr("King's Birthday")
            if self._year >= 2023
            # Queen's Birthday.
            else tr("Queen's Birthday")
        )
        dates_obs = {
            2022: (JUN, 3),
            2023: (JUN, 19),
            2024: (JUN, 17),
            2025: (JUN, 23),
        }
        if dt := dates_obs.get(self._year):
            self._add_holiday(name, dt)
        else:
            self._add_holiday_2nd_mon_of_jun(name)

        # Emancipation Day.
        self._add_observed(self._add_holiday_aug_1(tr("Emancipation Day")))

        if self._year >= 2014:
            # National Day of Thanksgiving.
            name = tr("National Day of Thanksgiving")
            if self._year >= 2026:
                self._add_holiday_1_day_prior_last_mon_of_aug(name)
            else:
                self._add_holiday_4th_fri_of_nov(name)

        if self._year >= 2026:
            # Constitution Day.
            self._add_holiday_last_mon_of_aug(tr("Constitution Day"))

        # National Youth Day.
        self._add_holiday_last_fri_of_sep(tr("National Youth Day"))

        self._add_holiday_2nd_mon_of_oct(
            # National Heritage Day.
            tr("National Heritage Day")
            if self._year >= 2014
            # Columbus Day.
            else tr("Columbus Day")
        )

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )


class TC(TurksAndCaicosIslands):
    pass


class TCA(TurksAndCaicosIslands):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import JAN, JUL
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SUN_TO_NEXT_MON,
    SUN_TO_NEXT_TUE,
)


class Barbados(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Barbados holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Barbados>
        * <https://web.archive.org/web/20250415140739/https://www.timeanddate.com/holidays/barbados>
        * [Public Holidays Act Cap.352](https://web.archive.org/web/20250427133553/https://barbadosparliament-laws.com/en/showdoc/cs/352)
        * <https://web.archive.org/web/20240629135051/https://labour.gov.bb/pdf/Library/Other%20Docs/Public%20Holidays%20for%20the%20Year%202018.pdf>
        * <https://web.archive.org/web/20240630134528/https://labour.gov.bb/wp-content/uploads/2020/04/Public-Holidays-for-the-Year-2021.pdf>
        * <https://web.archive.org/web/20241104070357/https://gisbarbados.gov.bb/download/public-holidays-for-2022/>
        * <https://web.archive.org/web/20230803161905/https://gisbarbados.gov.bb/download/public-holidays-for-2023/>
    """

    country = "BB"
    observed_label = "%s (observed)"
    # Public Holidays Act Cap.352, 1968-12-30
    start_year = 1969

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, BarbadosStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Errol Barrow Day
        if self._year >= 1989:
            self._add_observed(self._add_holiday_jan_21("Errol Barrow Day"))

        # Good Friday
        self._add_good_friday("Good Friday")

        # Easter Monday
        self._add_easter_monday("Easter Monday")

        # National Heroes Day
        if self._year >= 1998:
            self._add_observed(self._add_holiday_apr_28("National Heroes Day"))

        # May Day
        self._add_observed(self._add_labor_day("May Day"))

        # Whit Monday
        self._add_whit_monday("Whit Monday")

        # Emancipation Day
        name = "Emancipation Day"
        # If Aug 1 is Kadooment Day (i.e. Monday), observed on Tuesday.
        self._add_observed(self._add_holiday_aug_1(name), rule=SUN_TO_NEXT_TUE + MON_TO_NEXT_TUE)

        # Kadooment Day
        self._add_holiday_1st_mon_of_aug("Kadooment Day")

        # Independence Day
        self._add_observed(self._add_holiday_nov_30("Independence Day"))

        # Christmas
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SUN_TO_NEXT_TUE)

        # Boxing Day
        self._add_observed(self._add_christmas_day_two("Boxing Day"))


class BB(Barbados):
    pass


class BRB(Barbados):
    pass


class BarbadosStaticHolidays:
    special_public_holidays = {
        2021: (
            (JAN, 4, "Public Holiday"),
            (JAN, 5, "Public Holiday"),
        ),
        # One off 50th Anniversary of CARICOM Holiday.
        # See https://web.archive.org/web/20240805050828/https://gisbarbados.gov.bb/blog/one-off-bank-holiday-for-caricoms-50th-anniversary-celebrations/
        2023: (JUL, 31, "50th Anniversary of CARICOM Holiday"),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest
from datetime import date

from holidays.calendars.persian import _Persian


class TestPersianCalendar(unittest.TestCase):
    def setUp(self):
        super().setUpClass()
        self.calendar = _Persian()

    def test_year_bounds(self):
        self.assertIsNone(self.calendar.new_year_date(1900))
        self.assertIsNone(self.calendar.new_year_date(2101))
        self.assertIsNone(self.calendar.persian_to_gregorian(1900, 2, 2))
        self.assertIsNone(self.calendar.persian_to_gregorian(2101, 3, 3))

    def test_new_year_date(self):
        for year, day in (
            (2033, 20),
            (2066, 20),
            (2099, 20),
            (1904, 21),
            (1930, 21),
            (1961, 21),
            (1963, 21),
            (1992, 21),
            (2025, 21),
            (2058, 21),
            (2091, 21),
            (1922, 22),
            (1959, 22),
        ):
            self.assertEqual(self.calendar.new_year_date(year), date(year, 3, day))



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.afghanistan import Afghanistan
from tests.common import CommonCountryTests


class TestAfghanistan(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Afghanistan)

    def test_liberation_day(self):
        name = "روز آزادی"
        self.assertHolidayName(name, (f"{year}-02-15" for year in range(1989, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1989))

    def test_nowruz(self):
        name = "نوروز"
        self.assertHolidayName(
            name,
            "2011-03-21",
            "2012-03-20",
            "2013-03-21",
            "2014-03-21",
            "2015-03-21",
            "2016-03-20",
            "2017-03-21",
            "2018-03-21",
            "2019-03-21",
            "2020-03-20",
        )
        self.assertHolidayName(name, range(self.start_year, 1997), range(2001, 2021))
        self.assertNoHolidayName(name, range(1997, 2001), range(2021, self.end_year))

    def test_victory_of_mujahideen_day(self):
        name = "روز پیروزی مجاهدین"
        self.assertHolidayName(name, (f"{year}-04-28" for year in range(1992, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1992))

    def test_international_workers_day(self):
        name = "روز جهانی کارگر"
        self.assertHolidayName(
            name, (f"{year}-05-01" for year in (*range(1974, 1997), *range(2002, 2022)))
        )
        self.assertNoHolidayName(name, range(1997, 2002), range(2022, self.end_year))

    def test_soviet_victory_day(self):
        name = "روز پیروزی شوروی"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1978, 1989)))
        self.assertNoHolidayName(name, range(self.start_year, 1978), range(1989, self.end_year))

    def test_islamic_emirat_victory_day(self):
        name = "روز پیروزی امارت اسلامی"
        self.assertHolidayName(
            name,
            "2022-08-15",
            "2023-08-15",
            "2024-08-14",
        )
        self.assertHolidayName(name, range(2022, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2022))

    def test_american_withdrawal_day(self):
        name = "روز خروج آمریکایی ها"
        self.assertHolidayName(name, (f"{year}-08-31" for year in range(2022, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2022))

    def test_independence_day(self):
        self.assertHolidayName(
            "روز استقلال افغانستان", (f"{year}-08-19" for year in self.full_range)
        )

    def test_martyrs_day(self):
        name = "روز شهیدان"
        self.assertHolidayName(name, (f"{year}-09-09" for year in range(2012, 2021)))
        self.assertNoHolidayName(name, range(self.start_year, 2012), range(2021, self.end_year))

    def test_ashura(self):
        name = "عاشورا"
        self.assertHolidayName(
            name,
            "2014-11-03",
            "2015-10-24",
            "2016-10-12",
            "2017-10-01",
            "2018-09-21",
            "2019-09-10",
            "2020-08-30",
            "2021-08-19",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1924, 2022))
        self.assertNoIslamicNoEstimatedHolidayName(
            name, range(self.start_year, 1924), range(2022, self.end_year)
        )

    def test_prophets_birthday(self):
        name = "میلاد پیامبر"
        self.assertHolidayName(
            name,
            "2014-01-14",
            "2015-01-03",
            "2015-12-24",
            "2016-12-12",
            "2017-12-01",
            "2018-11-21",
            "2019-11-10",
            "2020-10-29",
            "2021-10-19",
            "2022-10-08",
            "2023-09-27",
            "2024-09-16",
            "2025-09-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1924, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1924))

    def test_first_day_of_ramadan(self):
        name = "اول رمضان"
        self.assertHolidayName(
            name,
            "2014-06-29",
            "2015-06-18",
            "2016-06-07",
            "2017-05-27",
            "2018-05-16",
            "2019-05-06",
            "2020-04-24",
            "2021-04-13",
            "2022-04-02",
            "2023-03-23",
            "2024-03-11",
            "2025-03-01",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1925, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1925))

    def test_eid_al_fitr(self):
        name = "عید فطر"
        self.assertHolidayName(
            name,
            "2014-07-29",
            "2014-07-30",
            "2014-07-31",
            "2015-07-18",
            "2015-07-19",
            "2015-07-20",
            "2016-07-07",
            "2016-07-08",
            "2016-07-09",
            "2017-06-26",
            "2017-06-27",
            "2017-06-28",
            "2018-06-15",
            "2018-06-16",
            "2018-06-17",
            "2019-06-04",
            "2019-06-05",
            "2019-06-06",
            "2020-05-24",
            "2020-05-25",
            "2020-05-26",
            "2021-05-13",
            "2021-05-14",
            "2021-05-15",
            "2022-05-01",
            "2022-05-02",
            "2022-05-03",
            "2023-04-22",
            "2023-04-23",
            "2023-04-24",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        years_eid_al_fitr_twice_all = {1935, 1968, 2000, 2033}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, set(range(1925, self.end_year)) - years_eid_al_fitr_twice_all
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, years_eid_al_fitr_twice_all)

    def test_arafah_day(self):
        name = "روز عرفه"
        self.assertHolidayName(
            name,
            "2014-10-04",
            "2015-09-22",
            "2016-09-12",
            "2017-09-01",
            "2018-08-21",
            "2019-08-10",
            "2020-07-30",
            "2021-07-19",
            "2022-07-08",
            "2023-06-27",
            "2024-06-16",
            "2025-06-06",
        )
        self.assertIslamicNoEstimatedHolidayName(name, range(1925, self.end_year))
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1925))

    def test_eid_al_adha(self):
        name = "عید قربانی"
        self.assertHolidayName(
            name,
            "2014-10-05",
            "2014-10-06",
            "2014-10-07",
            "2015-09-23",
            "2015-09-24",
            "2015-09-25",
            "2016-09-13",
            "2016-09-14",
            "2016-09-15",
            "2017-09-02",
            "2017-09-03",
            "2017-09-04",
            "2018-08-22",
            "2018-08-23",
            "2018-08-24",
            "2019-08-11",
            "2019-08-12",
            "2019-08-13",
            "2020-07-31",
            "2020-08-01",
            "2020-08-02",
            "2021-07-20",
            "2021-07-21",
            "2021-07-22",
            "2022-07-09",
            "2022-07-10",
            "2022-07-11",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-17",
            "2024-06-18",
            "2024-06-19",
            "2025-06-07",
            "2025-06-08",
            "2025-06-09",
        )
        years_eid_al_adha_twice_all = {1941, 1974, 2039}
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, set(range(1925, self.end_year)) - years_eid_al_adha_twice_all - {2006, 2007}
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 2006)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 5, 2007)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, years_eid_al_adha_twice_all)
        self.assertNoIslamicNoEstimatedHolidayName(name, range(self.start_year, 1925))

    def test_2021(self):
        self.assertHolidaysInYear(
            2021,
            ("2021-02-15", "روز آزادی"),
            ("2021-04-13", "اول رمضان"),
            ("2021-04-28", "روز پیروزی مجاهدین"),
            ("2021-05-01", "روز جهانی کارگر"),
            ("2021-05-13", "عید فطر"),
            ("2021-05-14", "عید فطر"),
            ("2021-05-15", "عید فطر"),
            ("2021-07-19", "روز عرفه"),
            ("2021-07-20", "عید قربانی"),
            ("2021-07-21", "عید قربانی"),
            ("2021-07-22", "عید قربانی"),
            ("2021-08-19", "روز استقلال افغانستان; عاشورا"),
            ("2021-10-19", "میلاد پیامبر"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-02-15", "روز آزادی"),
            ("2022-04-02", "اول رمضان"),
            ("2022-04-28", "روز پیروزی مجاهدین"),
            ("2022-05-01", "عید فطر"),
            ("2022-05-02", "عید فطر"),
            ("2022-05-03", "عید فطر"),
            ("2022-07-08", "روز عرفه"),
            ("2022-07-09", "عید قربانی"),
            ("2022-07-10", "عید قربانی"),
            ("2022-07-11", "عید قربانی"),
            ("2022-08-15", "روز پیروزی امارت اسلامی"),
            ("2022-08-19", "روز استقلال افغانستان"),
            ("2022-08-31", "روز خروج آمریکایی ها"),
            ("2022-10-08", "میلاد پیامبر"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-02-15", "Liberation Day"),
            ("2022-04-02", "First Day of Ramadan"),
            ("2022-04-28", "Mojahedin's Victory Day"),
            ("2022-05-01", "Eid al-Fitr"),
            ("2022-05-02", "Eid al-Fitr"),
            ("2022-05-03", "Eid al-Fitr"),
            ("2022-07-08", "Day of Arafah"),
            ("2022-07-09", "Eid al-Adha"),
            ("2022-07-10", "Eid al-Adha"),
            ("2022-07-11", "Eid al-Adha"),
            ("2022-08-15", "Islamic Emirate Victory Day"),
            ("2022-08-19", "Afghanistan Independence Day"),
            ("2022-08-31", "American Withdrawal Day"),
            ("2022-10-08", "Prophet's Birthday"),
        )

    def test_l10n_ps_af(self):
        self.assertLocalizedHolidays(
            "ps_AF",
            ("2022-02-15", "د ازادۍ ورځ"),
            ("2022-04-02", "د روژې لومړۍ نیټه"),
            ("2022-04-28", "مجاهدو د بریا ورځ"),
            ("2022-05-01", "عید فطر"),
            ("2022-05-02", "عید فطر"),
            ("2022-05-03", "عید فطر"),
            ("2022-07-08", "د عرفه ورځ"),
            ("2022-07-09", "عید قربانی"),
            ("2022-07-10", "عید قربانی"),
            ("2022-07-11", "عید قربانی"),
            ("2022-08-15", "د اسلامي امارت د بریا ورځ"),
            ("2022-08-19", "د افغانستان د استقلال ورځ"),
            ("2022-08-31", "د امریکا د وتلو ورځ"),
            ("2022-10-08", "د پیغمبر الله صلی الله علیه وسلم د میلاد ورځ"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.papua_new_guinea import PapuaNewGuinea
from tests.common import CommonCountryTests


class TestPapuaNewGuinea(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(
            PapuaNewGuinea, years=range(1953, 2050), years_non_observed=range(1953, 2050)
        )

    def test_special_holidays(self):
        dt = (
            "2018-11-16",
            "2021-01-08",
            "2021-03-01",
            "2021-03-12",
            "2022-09-19",
            "2023-04-18",
        )
        dt_observed = (
            "2022-02-28",
            "2023-02-24",
            "2023-06-16",
            "2023-09-15",
        )
        self.assertHoliday(dt, dt_observed)
        self.assertNoNonObservedHoliday(dt_observed)

    def test_1953(self):
        # http://www.paclii.org/pg/legis/consol_act/pha1953163.pdf
        self.assertHolidaysInYear(
            1953,
            ("1953-01-01", "New Year's Day"),
            ("1953-04-03", "Good Friday"),
            ("1953-04-04", "Easter Saturday"),
            ("1953-04-05", "Easter Sunday"),
            ("1953-04-06", "Easter Monday"),
            ("1953-06-08", "Queen's Birthday"),
            ("1953-07-23", "Papua New Guinea Remembrance Day"),
            ("1953-12-25", "Christmas Day"),
            ("1953-12-26", "Boxing Day"),
        )

    def test_2019(self):
        # https://www.scribd.com/document/465334129/PNG-2019-Gazetted-Public-Holidays-pdf
        # https://pngiportal.org/directory/national-gazette-g560-602-2018
        self.assertHolidaysInYear(
            2019,
            ("2019-01-01", "New Year's Day"),
            ("2019-04-19", "Good Friday"),
            ("2019-04-20", "Easter Saturday"),
            ("2019-04-21", "Easter Sunday"),
            ("2019-04-22", "Easter Monday"),
            ("2019-06-10", "Queen's Birthday"),
            ("2019-07-23", "Papua New Guinea Remembrance Day"),
            ("2019-08-26", "National Repentance Day"),
            ("2019-09-16", "Independence Day"),
            ("2019-12-25", "Christmas Day"),
            ("2019-12-26", "Boxing Day"),
        )

    def test_2020(self):
        # https://publicholidays.asia/wp-content/uploads/2020/01/PNG_PublicHolidays_2020.png
        self.assertHolidaysInYear(
            2020,
            ("2020-01-01", "New Year's Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-11", "Easter Saturday"),
            ("2020-04-12", "Easter Sunday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-06-08", "Queen's Birthday"),
            ("2020-07-23", "Papua New Guinea Remembrance Day"),
            ("2020-08-26", "National Repentance Day"),
            ("2020-09-16", "Independence Day"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
        )

    def test_2021(self):
        # https://publicholidays.asia/wp-content/uploads/2020/12/PNG_PublicHolidays_2021.pdf
        # https://www.efpng.org.pg/wp-content/uploads/2021/05/CIRCULAR-No.17-2021-AMENDMENTS-TO-2021-PUBLIC-HOLIDAY.pdf
        # Easter dates were later corrected as of https://web.archive.org/web/20210411125811/https://www.businessadvantagepng.com/papua-new-guinea-public-for-holidays/
        self.assertHolidaysInYear(
            2021,
            ("2021-01-01", "New Year's Day"),
            ("2021-01-08", "State Funeral of Sir Mekere Morauta"),
            ("2021-03-01", "National Day of Mourning for Sir Michael Somare"),
            ("2021-03-12", "National Day of Mourning for Sir Michael Somare"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-03", "Easter Saturday"),
            ("2021-04-04", "Easter Sunday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-06-14", "Queen's Birthday"),
            ("2021-07-23", "Papua New Guinea Remembrance Day"),
            ("2021-08-26", "National Repentance Day"),
            ("2021-09-16", "Independence Day"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Boxing Day (observed)"),
        )

    def test_2022(self):
        # https://publicholidays.asia/wp-content/uploads/2022/01/PNG_PublicHolidays_2022.pdf
        # Somare Day public holiday was observed on Feb 28 instead as of
        # https://web.archive.org/web/20220703204538/https://www.businessadvantagepng.com/papua-new-guinea-public-for-holidays/
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "New Year's Day"),
            ("2022-02-26", "Grand Chief Sir Michael Somare Remembrance Day"),
            ("2022-02-28", "Grand Chief Sir Michael Somare Remembrance Day (observed)"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-16", "Easter Saturday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-13", "Queen's Birthday"),
            ("2022-07-23", "Papua New Guinea Remembrance Day"),
            ("2022-08-26", "National Repentance Day"),
            ("2022-09-16", "Independence Day"),
            ("2022-09-19", "State Funeral of Queen Elizabeth II"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023(self):
        # https://publicholidays.asia/wp-content/uploads/2022/12/PNG_PublicHolidays_2023.pdf
        # https://pnghausbung.com/kings-birthday-holiday-to-be-observed-on-16th-june/
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-02-24", "Grand Chief Sir Michael Somare Remembrance Day (observed)"),
            ("2023-02-26", "Grand Chief Sir Michael Somare Remembrance Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-08", "Easter Saturday"),
            ("2023-04-09", "Easter Sunday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-04-18", "State Funeral of Sir Rabbie Namaliu"),
            ("2023-06-16", "King's Birthday (observed)"),
            ("2023-06-17", "King's Birthday"),
            ("2023-07-23", "Papua New Guinea Remembrance Day"),
            ("2023-07-24", "Papua New Guinea Remembrance Day (observed)"),
            ("2023-08-26", "National Repentance Day"),
            ("2023-09-15", "Independence Day (observed)"),
            ("2023-09-16", "Independence Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_2024(self):
        # https://web.archive.org/web/20231221053513/https://www.businessadvantagepng.com/papua-new-guinea-public-for-holidays/
        self.assertHolidaysInYear(
            2024,
            ("2024-01-01", "New Year's Day"),
            ("2024-02-26", "Grand Chief Sir Michael Somare Remembrance Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-03-31", "Easter Sunday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-06-17", "King's Birthday"),
            ("2024-07-23", "Papua New Guinea Remembrance Day"),
            ("2024-08-26", "National Repentance Day"),
            ("2024-09-16", "Independence Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import JAN, MAR, APR, MAY, JUN, AUG, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class SouthAfrica(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """South Africa holidays.

    References:
        * <https://web.archive.org/web/20250424073852/https://www.gov.za/about-sa/public-holidays>
        * [Act No. 36, 1994](https://web.archive.org/web/20250914045710/https://www.gov.za/sites/default/files/gcis_document/201409/act36of1994.pdf)
        * <https://web.archive.org/web/20250914050022/https://www.ancestors.co.za/origins-of-public-holidays-in-south-africa/>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_South_Africa>
        * <https://web.archive.org/web/20240715110800/https://www.gov.za/speeches/president-cyril-ramaphosa-progress-economic-recovery-30-oct-2023-0000>
        * <https://web.archive.org/web/20250427184315/https://www.gov.za/documents/notices/public-holidays-act-declaration-29-may-2024-public-holiday-23-feb-2024>
    """

    country = "ZA"
    # %s (observed).
    observed_label = "%s (observed)"
    # Public Holidays Act (No. 3 of 1910) came in effect on January 1st, 1911.
    start_year = 1911

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, SouthAfricaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 1995)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        if 1980 <= self._year <= 1994:
            # Founder's Day.
            self._add_holiday_apr_6("Founder's Day")
        elif 1952 <= self._year <= 1973:
            # Van Riebeeck's Day.
            self._add_holiday_apr_6("Van Riebeeck's Day")

        # Good Friday.
        self._add_good_friday("Good Friday")

        if self._year <= 1979:
            # Easter Monday.
            self._add_easter_monday("Easter Monday")

        # Family Day.
        name = "Family Day"
        if self._year >= 1980:
            self._add_easter_monday(name)
        elif 1961 <= self._year <= 1973:
            self._add_holiday_jul_10(name)

        # Workers' Day.
        name = "Workers' Day"
        if self._year >= 1995:
            self._add_observed(self._add_labor_day(name))
        elif 1987 <= self._year <= 1989:
            self._add_holiday_1st_fri_of_may(name)

        if self._year >= 1995:
            # Human Rights Day.
            self._add_observed(self._add_holiday_mar_21("Human Rights Day"))

            # Freedom Day.
            self._add_observed(self._add_holiday_apr_27("Freedom Day"))

            # Youth Day.
            self._add_observed(self._add_holiday_jun_16("Youth Day"))

            # National Women's Day.
            self._add_observed(self._add_holiday_aug_9("National Women's Day"))

            # Heritage Day.
            self._add_observed(self._add_holiday_sep_24("Heritage Day"))
        elif self._year <= 1993:
            self._add_holiday_may_31(
                # Republic Day.
                "Republic Day"
                if self._year >= 1961
                # Union Day.
                else "Union Day"
            )

            # Ascension Day.
            self._add_ascension_thursday("Ascension Day")

            if self._year >= 1952:
                # Kruger Day.
                self._add_holiday_oct_10("Kruger Day")

        if self._year >= 1952:
            if self._year <= 1960:
                # Queen's Birthday.
                self._add_holiday_2nd_mon_of_jul("Queen's Birthday")

            if self._year <= 1979:
                # Settlers' Day.
                self._add_holiday_1st_mon_of_sep("Settlers' Day")
        else:
            # According to our sources, Act No. 3 of 1910 supposedly uses the term "Victoria Day"
            # but as we can't find the original document and that Empire Day was supposed
            # instituted empire-wide circa 1904 - we'll stick with its latter name for now.

            # Empire Day.
            self._add_holiday_may_24("Empire Day")

            # King's Birthday.
            self._add_holiday_1st_mon_of_aug("King's Birthday")

        if self._year >= 1995:
            # Day of Reconciliation.
            name = "Day of Reconciliation"
        elif self._year >= 1980:
            # Day of the Vow.
            name = "Day of the Vow"
        elif self._year >= 1952:
            # Day of the Covenant.
            name = "Day of the Covenant"
        else:
            # Dingaan's Day.
            name = "Dingaan's Day"
        self._add_observed(self._add_holiday_dec_16(name))

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        self._add_observed(
            self._add_christmas_day_two(
                # Day of Goodwill.
                "Day of Goodwill"
                if self._year >= 1980
                # Boxing Day.
                else "Boxing Day"
            )
        )


class ZA(SouthAfrica):
    pass


class ZAF(SouthAfrica):
    pass


class SouthAfricaStaticHolidays:
    # Local Government Elections.
    local_elections = "Local Government Elections"
    # National and Provincial Government Elections.
    national_and_provincial_elections = "National and Provincial Government Elections"
    # Public Holiday by Presidential Decree.
    presidential_decree_holiday = "Public Holiday by Presidential Decree"
    # Y2K Changeover.
    y2k_changeover = "Y2K Changeover"
    special_public_holidays = {
        1999: (
            (JUN, 2, national_and_provincial_elections),
            (DEC, 31, y2k_changeover),
        ),
        2000: (JAN, 2, y2k_changeover),
        2004: (APR, 14, national_and_provincial_elections),
        2006: (MAR, 1, local_elections),
        2008: (MAY, 2, presidential_decree_holiday),
        2009: (APR, 22, national_and_provincial_elections),
        2011: (
            (MAY, 18, local_elections),
            (DEC, 27, presidential_decree_holiday),
        ),
        2014: (MAY, 7, national_and_provincial_elections),
        2016: (
            (AUG, 3, local_elections),
            (DEC, 27, presidential_decree_holiday),
        ),
        2019: (MAY, 8, national_and_provincial_elections),
        # Municipal elections.
        2021: (NOV, 1, "Municipal elections"),
        2022: (DEC, 27, presidential_decree_holiday),
        2023: (DEC, 15, presidential_decree_holiday),
        2024: (MAY, 29, national_and_provincial_elections),
    }
    special_public_holidays_observed = {
        # https://web.archive.org/web/20120328122217/http://www.info.gov.za/speeches/1999/991028409p1002.htm
        2000: (JAN, 3, y2k_changeover),
    }
